/* Cluster.java

	Purpose:
		
	Description:
		
	History:
		2:11 PM 2022/9/8, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;

import org.zkoss.chart.plotOptions.DataLabels;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * Options for marker clusters, the concept of sampling the data values into larger
 * blocks in order to ease readability and increase performance of the JavaScript charts.
 *
 * <p>
 * Note: marker clusters module is not working with boost and draggable-points modules.
 *
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class Cluster extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		allowOverlap,
		animation,
		dataLabels,
		drillToCluster,
		enabled,
		//TODO
		events,
		layoutAlgorithm,
		marker,
		minimumClusterSize,
		states,
		zones
	}
	/**
	 * Returns false to prevent cluster overlapping - this option works only
	 * when {@code layoutAlgorithm.type} = "grid".
	 * <p>
	 * Default: true.
	 */
	public boolean isAllowOverlap() {
		return getAttr(Attrs.allowOverlap, true).asBoolean();
	}
	/**
	 * Sets to false prevent cluster overlapping - this option works only when
	 * {@code layoutAlgorithm.type} = "grid".
	 * <p>
	 * Default: true.
	 */
	public void setAllowOverlap(Boolean allowOverlap) {
		setAttr(Attrs.allowOverlap, allowOverlap);
	}



	/**
	 * Returns whether to enable or disable for the cluster marker animation.
	 * <p>Default: true
	 */
	public boolean isAnimation() {
		return getAnimation() != Animation.NONE;
	}

	/**
	 * Returns whether to enable or disable for the cluster marker animation.
	 * The animation can also be set as a {@link Animation}
	 * object.
	 * <p>
	 * Default: {@code {"duration": 500}}
	 */
	public Animation getAnimation() {
		if (containsKey(Attrs.animation)) {
			return (Animation) getAttr(Attrs.animation);
		} else {
			Animation defaultAnimation = new Animation();
			defaultAnimation.setDuration(500);
			return (Animation) getAttr(Attrs.animation, defaultAnimation).asValue();
		}
	}

	/**
	 * Sets whether to enable or disable for the cluster marker animation.
	 *
	 * @see #setAnimation(Animation)
	 * @see Animation
	 */
	public void setAnimation(boolean animation) {
		setAnimation(animation ? new Animation() : Animation.NONE);
	}

	/**
	 * Sets whether to enable or disable for the cluster marker animation.
	 * The animation can also be set as a {@link Animation}
	 * object.
	 * <p>
	 * Default: {@code {"duration": 500}}
	 *
	 * @see #setAnimation(boolean)
	 * @see Animation
	 */
	public void setAnimation(Animation animation) {
		setAttr(Attrs.animation, animation == null ? Animation.NONE : animation);
	}

	/**
	 * Returns the cluster data labels.
	 * <p>Default: an instance of DataLabels.
	 */
	public DataLabels getDataLabels() {
		return (DataLabels) getAttr(Attrs.dataLabels, DataLabels.class).asValue();
	}

	/**
	 * Sets the cluster data labels.
	 *
	 * @see DataLabels
	 */
	public void setDataLabels(DataLabels dataLabels) {
		setAttr(Attrs.dataLabels, dataLabels);
	}

	/**
	 * Returns whether to zoom the plot area to the cluster points range
	 * when a cluster is clicked.
	 * <p>Default: true.
	 */
	public boolean isDrillToCluster() {
		return getAttr(Attrs.drillToCluster, true).asBoolean();
	}

	/**
	 * Sets whether to zoom the plot area to the cluster points range
	 * when a cluster is clicked.
	 * <p>Default: true.
	 */
	public void setDrillToCluster(boolean drillToCluster) {
		setAttr(Attrs.drillToCluster, drillToCluster);
	}

	/**
	 * Returns whether to enable the marker-clusters module.
	 * <p>Default: false.
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, true).asBoolean();
	}

	/**
	 * Sets whether to enable the marker-clusters module.
	 * <p>Default: false.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}

	/**
	 * Options for layout algorithm. Inside there are options to change the type of
	 * the algorithm, gridSize, distance or iterations.
	 * <p>Default: an instance of ClusterLayoutAlgorithm
	 */
	public ClusterLayoutAlgorithm getLayoutAlgorithm() {
		return (ClusterLayoutAlgorithm) getAttr(Attrs.layoutAlgorithm, ClusterLayoutAlgorithm.class).asValue();
	}

	/**
	 * Options for layout algorithm. Inside there are options to change the type of
	 * the algorithm, gridSize, distance or iterations.
	 * <p>Default: null
	 */
	public void setLayoutAlgorithm(ClusterLayoutAlgorithm layoutAlgorithm) {
		setAttr(Attrs.layoutAlgorithm, layoutAlgorithm);
	}

	/**
	 * Returns the cluster marker.
	 * <p>Default: an instance of Marker
	 */
	public Marker getMarker() {
		return (Marker) getAttr(Attrs.marker, Marker.class).asValue();
	}

	/**
	 * Sets the cluster marker
	 */
	public void setMarker(Marker marker) {
		setAttr(Attrs.marker, marker);
	}

	/**
	 * The minimum amount of points to be combined into a cluster.
	 * This value has to be greater or equal to 2.
	 * <p>
	 * Defaults to 2.
	 */
	public Number getMinimumClusterSize() {
		return getAttr(Attrs.minimumClusterSize, 2).asNumber();
	}

	/**
	 * The minimum amount of points to be combined into a cluster.
	 * This value has to be greater or equal to 2.
	 * <p>
	 * Defaults to 2.
	 */
	public void setMinimumClusterSize(Number minimumClusterSize) {
		setAttr(Attrs.minimumClusterSize, minimumClusterSize);
	}

	/**
	 * Returns a wrapper object for the cluster options in specific states.
	 * <p>Default: an instance of States
	 */
	public States getStates() {
		return (States) getAttr(Attrs.states, States.class).asValue();
	}

	/**
	 * Sets a wrapper object for the cluster options in specific states.
	 */
	public void setStates(States states) {
		setAttr(Attrs.states, states);
	}



	/**
	 * An list defining zones within marker clusters.
	 * <p>Default: null.
	 * @see Zone
	 */
	public List<Zone> getZones() {
		return Generics.cast(getAttr(Attrs.zones, null).asValue());
	}

	/**
	 * Set the zones within marker clusters.
	 * @see #getZones()
	 * @see Zone
	 */
	public void setZones(List<Zone> zones) {
		setAttr(Attrs.zones, zones);
	}

	/**
	 * Set the zones within marker clusters.
	 * @see #getZones()
	 * @see Zone
	 */
	public void setZones(Zone... zones) {
		Objects.requireNonNull(zones);
		setZones(Arrays.asList(zones));
	}
}
