/* Drilldown.java

	Purpose:
		
	Description:
		
	History:
		Tue, Jan 28, 2014 12:21:12 PM, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.

 */
package org.zkoss.chart;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.lang.Generics;

/**
 * Options for drill down, the concept of inspecting increasingly high
 * resolution data through clicking on chart items like columns or pie slices.
 * 
 * <p> All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author jumperchen
 * @author RaymondChao
 */
public class Drilldown extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		activeAxisLabelStyle,
		activeDataLabelStyle,
		animation,
		breadcrumbs,
		drillUpButton,
		series
	}

	public Drilldown() {
		// Prevent null
		setSeries(new ArrayList<Series>());
	}

	/**
	 * Returns additional styles to apply to the X axis label for a point that
	 * has drilldown data. By default it is underlined and blue to invite to
	 * interaction. Defaults to:
	 * <pre>
	 * cursor: pointer; color: #0d233a; fontWeight: bold; textDecoration: underline;
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setActiveAxisLabelStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setActiveAxisLabelStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setActiveAxisLabelStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setActiveAxisLabelStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getActiveAxisLabelStyle() {
		if (!containsKey(Attrs.activeAxisLabelStyle)) {
			setActiveAxisLabelStyle("cursor: pointer; color: #0d233a; fontWeight: bold; textDecoration: underline;");
		}
		return Generics.cast(getAttr(Attrs.activeAxisLabelStyle));
	}

	/**
	 * Sets additional styles to apply to the X axis label for a point that has
	 * drilldown data. By default it is underlined and blue to invite to
	 * interaction.<p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param labelStyle the labelStyle object
	 */
	public <K, V> void setActiveAxisLabelStyle(Map<K, V> labelStyle) {
		setAttr(Attrs.activeAxisLabelStyle, labelStyle, NOT_NULL_VALUE);
	}

	/**
	 * Sets additional styles to apply to the X axis label for a point that has
	 * drilldown data. By default it is underlined and blue to invite to
	 * interaction.<p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setActiveAxisLabelStyle(Map)}
	 * @param activeAxisLabelStyle one or many CSS key:value pairs in a single line string format
	 */
	public void setActiveAxisLabelStyle(String activeAxisLabelStyle) {
		setActiveAxisLabelStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), activeAxisLabelStyle, ':',
				';', '\''));
	}

	/**
	 * Returns additional styles to apply to the data label of a point that has
	 * drilldown data. By default it is underlined and blue to invite to
	 * interaction. Defaults to:
	 * 
	 * <pre>
	 * cursor: pointer; color: #0d233a; fontWeight: bold; textDecoration: underline;
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setActiveDataLabelStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setActiveDataLabelStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setActiveDataLabelStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setActiveDataLabelStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getActiveDataLabelStyle() {
		if (!containsKey(Attrs.activeDataLabelStyle)) {
			setActiveDataLabelStyle("cursor: pointer; color: #0d233a; fontWeight: bold; textDecoration: underline;");
		}
		return Generics.cast(getAttr(Attrs.activeDataLabelStyle));
	}

	/**
	 * Sets additional styles to apply to the data label of a point that has
	 * drilldown data. By default it is underlined and blue to invite to
	 * interaction.<p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setActiveDataLabelStyle(Map)}
	 * @param activeDataLabelStyle one or many CSS key:value pairs in a single line string format
	 */
	public void setActiveDataLabelStyle(String activeDataLabelStyle) {
		setActiveDataLabelStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), activeDataLabelStyle, ':',
				';', '\''));
	}

	/**
	 * Sets additional styles to apply to the data label of a point that has
	 * drilldown data. By default it is underlined and blue to invite to
	 * interaction.<p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param activeDataLabelStyle the style object
	 */
	public <K, V> void setActiveDataLabelStyle(Map<K, V> activeDataLabelStyle) {
		setAttr(Attrs.activeDataLabelStyle, activeDataLabelStyle, NOT_NULL_VALUE);
	}

	/**
	 * Returns the overall animation for all drilldown updating.
	 * <p>
	 * Default: an instance of {@link Animation}
	 */
	public Animation getAnimation() {
		return (Animation) getAttr(Attrs.animation,
				org.zkoss.chart.Animation.class).asValue();
	}

	/**
	 * Returns whether enable the animation
	 * <p>
	 * Default: true
	 */
	public boolean isAnimation() {
		return getAnimation() != Animation.NONE;
	}

	/**
	 * Sets the animation for all drilldown animations. Animation of a drilldown
	 * occurs when drilling between a column point and a column series, or a pie
	 * slice and a full pie series. Drilldown can still be used between series
	 * and points of different types, but animation will not occur.
	 * <p>
	 * The animation can either be set as a boolean or a configuration object.
	 * See {@link #setAnimation(Animation)}. If <code>true</code>, it will use
	 * the 'swing' jQuery easing and a duration of 500 ms. If used as an
	 * Animation object, the following properties are supported:
	 * </p>
	 * <dl>
	 * <dt>duration</dt>
	 * <dd>The duration of the animation in milliseconds.</dd>
	 * <dt>easing</dt>
	 * <dd>When using jQuery as the general framework, the easing can be set to
	 * <code>linear</code> or <code>swing</code>. More easing functions are
	 * available with the use of jQuery plug-ins, most notably the jQuery UI
	 * suite. See <a href="http://api.jquery.com/animate/">the jQuery docs</a>.
	 * When using MooTools as the general framework, use the property name
	 * <code>transition</code> instead of <code>easing</code>.</dd>
	 * </dl>
	 * <p>
	 * 
	 * @see #setAnimation(Animation)
	 * @see Animation
	 */
	public void setAnimation(boolean animation) {
		setAnimation(animation ? new Animation() : Animation.NONE);
	}

	/**
	 * Sets the animation for all drilldown animations. Animation of a drilldown
	 * occurs when drilling between a column point and a column series, or a pie
	 * slice and a full pie series. Drilldown can still be used between series
	 * and points of different types, but animation will not occur.
	 * <p>
	 * Properties are supported:
	 * <dl>
	 * <dt>duration</dt>
	 * <dd>The duration of the animation in milliseconds.</dd>
	 * <dt>easing</dt>
	 * <dd>When using jQuery as the general framework, the easing can be set to
	 * <code>linear</code> or <code>swing</code>. More easing functions are
	 * available with the use of jQuery plug-ins, most notably the jQuery UI
	 * suite. See <a href="http://api.jquery.com/animate/">the jQuery docs</a>.
	 * When using MooTools as the general framework, use the property name
	 * <code>transition</code> instead of <code>easing</code>.</dd>
	 * </dl>
	 * <p>
	 * 
	 * @see #setAnimation(boolean)
	 * @see Animation
	 */
	public void setAnimation(Animation animation) {
		setAttr(Attrs.animation, animation == null ? Animation.NONE : animation);
	}

	/**
	 * Returns options for the breadcrumbs, the navigation at the top leading the
	 * way up through the drilldown levels. If null, create a new one.
	 * @since 10.2.1.0
	 */
	public Breadcrumbs getBreadcrumbs() {
		return (Breadcrumbs) getAttr(Attrs.breadcrumbs, Breadcrumbs.class).asValue();
	}

	/**
	 * Sets options for the breadcrumbs, the navigation at the top leading the way up through the drilldown levels.
	 * @since 10.2.1.0
	 */
	public void setBreadcrumbs(Breadcrumbs breadcrumbs) {
		setAttr(Attrs.breadcrumbs, breadcrumbs);
	}

	/**
	 * Returns options for the drill up button that appears when drilling down
	 * on a series. The text for the button is defined in
	 * {@link Lang#getDrillUpText()}.
	 * 
	 * @see DrillUpButton
	 */
	public DrillUpButton getDrillUpButton() {
		return (DrillUpButton) getAttr(Attrs.drillUpButton, null).asValue();
	}

	/**
	 * Sets options for the drill up button that appears when drilling down on a
	 * series. The text for the button is defined in
	 * {@link Lang#setDrillUpText(String)}.
	 * 
	 * @see DrillUpButton
	 */
	public void setDrillUpButton(DrillUpButton drillUpButton) {
		setAttr(Attrs.drillUpButton, drillUpButton);
	}

	/**
	 * Returns an array of series configurations for the drill down. Each series
	 * configuration uses the same syntax as {@link Series}. These drilldown
	 * series are hidden by default. The drilldown series is linked to the
	 * parent series' point by its {@link Series#setId(String)}.
	 */
	public List<Series> getSeries() {
		return Generics.cast(getAttr(Attrs.series));
	}

	/**
	 * Sets an array of series configurations for the drill down. Each series
	 * configuration uses the same syntax as {@link Series}. These drilldown
	 * series are hidden by default. The drilldown series is linked to the
	 * parent series' point by its {@link Series#setId(String)}.
	 */
	public void setSeries(List<Series> series) {
		setAttr(Attrs.series, series);
	}

	/**
	 * Sets an array of series configurations for the drill down. Each series
	 * configuration uses the same syntax as {@link Series}. These drilldown
	 * series are hidden by default. The drilldown series is linked to the
	 * parent series' point by its {@link Series#setId(String)}.
	 * @see #setSeries(List)
	 */
	public void setSeries(Series... series) {
		setSeries(Arrays.asList(series));
	}

}