/** Loading.java.

	Purpose:
		
	Description:
		
	History:
		10:10:46 AM Jan 28, 2014, Created by jumperchen

Copyright (C) 2014 Potix Corporation. All Rights Reserved.
 */
package org.zkoss.chart;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.lang.Generics;

/**
 * The loading options control the appearance of the loading screen that covers
 * the plot area on chart operations. This screen only appears after an explicit
 * call to {@link Charts#showLoading(String)}.
 * 
 * <p> All the options in this class support {@link DynamicalAttribute}.
 * @author jumperchen
 */
public class Loading extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		hideDuration, labelStyle, showDuration, style
	}

	/**
	 * Sets the duration in milliseconds of the fade out effect.
	 * <p>
	 * Defaults to 100.
	 * 
	 * @param hideDuration
	 */
	public void setHideDuration(Number hideDuration) {
		setAttr(Attrs.hideDuration, hideDuration, 100);
	}

	/**
	 * Returns the duration in milliseconds of the fade out effect.
	 * <p>
	 * Defaults to 100.
	 */
	public Number getHideDuration() {
		return getAttr(Attrs.hideDuration, 100).asNumber();
	}

	/**
	 * Sets CSS styles for the loading label span.<p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * loading.setLabelStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setLabelStyle(Map)}
	 * @param labelStyle one or many CSS key:value pairs in a single line string format
	 */
	public void setLabelStyle(String labelStyle) {
		setLabelStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), labelStyle, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the loading label span.<p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param labelStyle the style object
	 */
	public <K, V> void setLabelStyle(Map<K, V> labelStyle) {
		setAttr(Attrs.labelStyle, labelStyle);
	}

	/**
	 * Returns CSS styles for the loading label span. Defaults to
	 * <pre>
	 * fontWeight: 'bold'; position: 'relative'; top: '45%';
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setLabelStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setLabelStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setLabelStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setLabelStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getLabelStyle() {
		if (!containsKey(Attrs.style)) {
			setStyle("fontWeight: 'bold'; position: 'relative'; top: '45%'");
		}
		return Generics.cast(getAttr(Attrs.labelStyle));
	}

	/**
	 * Sets the duration in milliseconds of the fade in effect.
	 * <p>
	 * Defaults to 100.
	 * 
	 * @param showDuration
	 */
	public void setShowDuration(Number showDuration) {
		setAttr(Attrs.showDuration, showDuration, 100);
	}

	/**
	 * Returns the duration in milliseconds of the fade in effect.
	 * <p>
	 * Defaults to 100.
	 */
	public Number getShowDuration() {
		return getAttr(Attrs.showDuration, 100).asNumber();
	}

	/**
	 * Returns CSS styles for the label. When labels are rotated they are
	 * rendered using vector graphic techniques and not all styles are
	 * applicable. Default:
	 * 
	 * <pre>
	 * position: 'absolute'; backgroundColor: 'white'; opacity: 0.5; textAlign: 'center'
	 * </pre>
	 * <p>
	 * 
	 * @return style CSS styles for the label
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getStyle() {
		if (!containsKey(Attrs.style)) {
			setStyle("position: 'absolute'; backgroundColor: 'white'; opacity: 0.5; textAlign: 'center'");
		}
		return Generics.cast(getAttr(Attrs.style));
	}

	/**
	 * Sets CSS styles for the loading screen that covers the plot area.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 */
	public void setStyle(String style) {
		setStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the loading screen that covers the plot area.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style);
	}
}
