/* Point.java

	Purpose:
		
	Description:
		
	History:
		Thu, Jan 16, 2014  2:39:56 PM, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.

 */
package org.zkoss.chart;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.zkoss.chart.OptionDataEvent.EventType;
import org.zkoss.chart.plotOptions.DataLabels;
import org.zkoss.chart.plotOptions.DragDrop;
import org.zkoss.chart.plotOptions.GaugeDialPlotOptions;
import org.zkoss.chart.plotOptions.TargetOptions;
import org.zkoss.chart.plotOptions.XRangePartialFill;
import org.zkoss.chart.util.DeferredCall;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.JSFunction;
import org.zkoss.chart.util.OptionsList;
import org.zkoss.json.JSONArray;
import org.zkoss.json.JSONObject;
import org.zkoss.json.JSONValue;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.lang.Generics;

/**
 * The Point object can be accessed in a number of ways. In the {@link Series}
 * object all the points are accessed by the {@link Series#setData(Double...)}.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author jumperchen
 * @author RaymondChao
 */
public class Point extends Optionable {
	private enum Attrs implements PlotAttribute {
		selected,
		sliced
	}
	private enum DAttrs implements PlotAttribute, DynamicalAttribute {
		accessibility,
		color,
		colorIndex,
		dataLabels,
		dashStyle,
		description,
		direction,
		dragDrop,
		events,
		from,
		id,
		innerRadius,
		label,
		labelrank,
		length,
		legendIndex,
		marker,
		name,
		outgoing,
		partialFill,
		pointWidth,
		radius,
		selected,
		sets,
		sliced,
		target,
		targetOptions,
		to,
		weight,
		width,
		x,
		x2,
		y,
		z,
		dial,
		drilldown,

		isIntermediateSum,
		isSum,
		visible,

		colorValue,
		parent,
		value,
		
		q1,
		median,
		q3,
		
		// extra DAttrs internal use only
		low,
		high
	}

	private boolean _valuesOnly = true;
	private Series _series;

	/**
	 * Default constructor
	 */
	public Point() {
	}

	/* package */void setSeries(Series s) {
		_series = s;
	}

	/* package */Series getSeries() {
		return _series;
	}

	/**
	 * Constructs with a number
	 */
	public Point(Number y) {
		setY(y);
	}

	/**
	 * Constructs with a pair number (x, y)
	 */
	public Point(Number x, Number y) {
		setX(x);
		setY(y);
	}

	/**
	 * Constructs with a name and number
	 * 
	 * @param name
	 * @param y
	 */
	public Point(String name, Number y) {
		setName(name);
		setY(y);
	}

	/**
	 * Constructs with a name, Y number, and X index
	 * 
	 * @param name
	 * @param y
	 * @param x
	 * @since 1.1
	 */
	public Point(String name, Number y, Number x) {
		setName(name);
		setY(y);
		setX(x);
	}

	/**
	 * Constructs with a name, number, and the id of the drilldown data
	 * 
	 * @param name
	 * @param y
	 * @param drilldown
	 */
	public Point(String name, Number y, String drilldown) {
		setName(name);
		setY(y);
		setDrilldown(drilldown);
	}

	/**
	 * Constructs with three numbers, x, low, and high.
	 */
	public Point(Number x, Number low, Number high) {
		setX(x);
		setLow(low);
		setHigh(high);
	}

	/**
	 * Constructs with from and to. Suitable for networkgraph.
	 * @since 7.2.1.0
	 */
	public Point(String from, String to) {
		setFrom(from);
		setTo(to);
	}

	/**
	 * Constructs with from, to and weight. Suitable for sankey.
	 * @since 7.2.1.0
	 */
	public Point(String from, String to, Number weight) {
		setFrom(from);
		setTo(to);
		setWeight(weight);
	}

	/**
	 * Constructs with id, parent and name. Suitable for treegraph.
	 * @since 11.2.0.0
	 */
	public Point(String id, String parent, String name) {
		setId(id);
		setParent(parent);
		setName(name);
	}

	/**
	 * Constructs with three numbers, x, y, and z.
	 * @return a new Point
	 * @since 7.2.1.2
	 */
	public static Point ofXYZ(Number x, Number y, Number z) {
		final Point point = new Point(x, y);
		point.setZ(z);
		return point;
	}

	/**
	 * Constructs with name and low value.
	 * @since 10.2.1.0
	 */
	public static Point ofLow(String name, Number low) {
		Point point = new Point();
		point.setName(name);
		point.setLow(low);
		return point;
	}

	/**
	 * Constructs with name, low, and high value.
	 * @since 10.2.1.0
	 */
	public static Point ofLowHigh(String name, Number low, Number high) {
		Point point = new Point();
		point.setName(name);
		point.setLow(low);
		point.setHigh(high);
		return point;
	}

	/**
	 * Returns a point with an array like data structure.
	 * @since 10.2.1.0
	 */
	public static <T> Point of(T... any) {
		return new ArrayPoint(any);
	}

	/**
	 * Returns a point with a given map data structure.
	 * <p>Note: each of the key is not a dynamical attribute, so it won't trigger
	 * a change event.</p>
	 * @since 10.2.1.0
	 */
	public static <T> Point ofMap(Map<String, T> map) {
		Point point = new Point();
		point._valuesOnly = false;
		for (Map.Entry<String, T> me: map.entrySet()) {
			point.setAttr(me.getKey(), me.getValue(), NOT_NULL_VALUE);
		}
		return point;
	}

	/**
	 * Returns the first or single option of data labels.
	 */
	@SuppressWarnings("unchecked")
	public DataLabels getDataLabels() {
		Object dataLabelsObject = getAttr(DAttrs.dataLabels);

		if (dataLabelsObject == null) {
			DataLabels dataLabels = new DataLabels();
			setDataLabels(dataLabels);
			return dataLabels;
		}
		if (dataLabelsObject instanceof List<?>) {
			return ((List<DataLabels>) dataLabelsObject).get(0);
		}
		return (DataLabels) dataLabelsObject;
	}



	/**
	 * Returns an accessibility options for a data point, if null, it will create new one.
	 * @since 10.2.1.0
	 */
	public PointAccessibility getAccessibility() {
		return (PointAccessibility) getAttr(DAttrs.accessibility, PointAccessibility.class).asValue();
	}

	/**
	 * Sets an accessibility options for a data point.
	 * @since 10.2.1.0
	 */
	public void setAccessibility(PointAccessibility accessibility) {
		setAttr(DAttrs.accessibility, accessibility);
	}

	/**
	 * Sets the first or single option of data labels.
	 *
	 * @see DataLabels
	 */
	public void setDataLabels(DataLabels dataLabels) {
		_valuesOnly = false;
		setAttr(DAttrs.dataLabels, dataLabels);
	}

	/**
	 * Returns the list of options of data labels.
	 * @since 7.2.2.0
	 */
	@SuppressWarnings("unchecked")
	public List<DataLabels> getDataLabelsList() {
		Object dataLabelsObject = getAttr(DAttrs.dataLabels);

		if (dataLabelsObject == null) {
			List<DataLabels> dataLabels = new ArrayList<DataLabels>();
			setDataLabels(dataLabels);
			return dataLabels;
		}
		if (dataLabelsObject instanceof DataLabels) {
			List<DataLabels> dataLabels = new ArrayList<DataLabels>(1);
			dataLabels.add((DataLabels) dataLabelsObject);
			setDataLabels(dataLabels);
			return dataLabels;
		}
		return (List<DataLabels>) dataLabelsObject;
	}

	/**
	 * Sets the list of options of data labels.
	 *
	 * @see DataLabels
	 * @since 7.2.2.0
	 */
	public void setDataLabels(List<DataLabels> dataLabels) {
		_valuesOnly = false;
		setAttr(DAttrs.dataLabels, dataLabels);
	}

	/**
	 * Add an option of data labels to the list.
	 *
	 * @see DataLabels
	 * @since 7.2.2.0
	 */
	public void addDataLabels(DataLabels dataLabels) {
		getDataLabelsList().add(dataLabels);
	}

	/**
	 * Returns a name for the dash style to use for the graph, or for some series types the outline of each shape.
	 * <p>
	 * Default: "Solid"
	 * @since 7.2.1.0
	 */
	public String getDashStyle() {
		return getAttr(DAttrs.dashStyle, "Solid").asString();
	}

	/**
	 * Sets a name for the dash style to use for the graph, or for some series types the outline of each shape.
	 * @since 7.2.1.0
	 */
	public void setDashStyle(String dashStyle) {
		_valuesOnly = false;
		setAttr(DAttrs.dashStyle, dashStyle);
	}

	/**
	 * Returns the description. This description will be shown in tooltip.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public String getDescription() {
		return getAttr(DAttrs.description, null).asString();
	}

	/**
	 * Sets the description. This description will be shown in tooltip.
	 * @since 7.2.1.0
	 */
	public void setDescription(String description) {
		_valuesOnly = false;
		setAttr(DAttrs.description, description);
	}

	/**
	 * Returns the vector direction in degrees, where 0 is north (pointing towards south).
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Number getDirection() {
		return getAttr(DAttrs.direction, null).asNumber();
	}

	/**
	 * Sets the vector direction in degrees, where 0 is north (pointing towards south).
	 * @since 7.2.1.0
	 */
	public void setDirection(Number direction) {
		_valuesOnly = false;
		setAttr(DAttrs.direction, direction);
	}

	/**
	 * Returns the options of point specific options for the draggable-points module, if any.
	 * Otherwise, create a new one.
	 * @since 7.2.1.0
	 */
	public DragDrop getDragDrop() {
		DragDrop dragDrop = (DragDrop) getAttr(DAttrs.dragDrop);
		if (dragDrop == null) {
			dragDrop = new DragDrop();
			setDragDrop(dragDrop);
		}
		return dragDrop;
	}

	/**
	 * Sets the options of point specific options for the draggable-points module.
	 * @since 7.2.1.0
	 */
	public void setDragDrop(DragDrop dragDrop) {
		_valuesOnly = false;
		setAttr(DAttrs.dragDrop, dragDrop);
	}

	/**
	 * Returns whether the series is visible or not.
	 * <p>
	 * Default: true
	 */
	public boolean isVisible() {
		return getAttr(DAttrs.visible, true).asBoolean();
	}

	/**
	 * Sets whether the serries is visible.
	 */
	public void setVisible(final boolean visible) {
		_valuesOnly = false;
		setAttr(DAttrs.visible, visible, true);
	}

	/**
	 * Sets the low number, if any.
	 */
	public void setLow(Number low) {
		setAttr(DAttrs.low, low, NOT_NULL_VALUE);
	}

	/**
	 * Returns the low number, if any.
	 * <p>
	 * Default: null
	 */
	public Number getLow() {
		return getAttr(DAttrs.low, null).asNumber();
	}

	/**
	 * Sets the high number, if any.
	 */
	public void setHigh(Number high) {
		setAttr(DAttrs.high, high, NOT_NULL_VALUE);
	}

	/**
	 * Returns the high number, if any.
	 * <p>
	 * Default: null
	 */
	public Number getHigh() {
		return getAttr(DAttrs.high, null).asNumber();
	}

	/**
	 * Returns the dial data for gauge chart type only.
	 */
	public GaugeDialPlotOptions getDial() {
		GaugeDialPlotOptions dial = (GaugeDialPlotOptions) getAttr(DAttrs.dial);
		if (dial == null) {
			dial = new GaugeDialPlotOptions();
			setDial(dial);
		}
		return dial;
	}

	/**
	 * Sets the dial data for gauge chart type only.
	 */
	public void setDial(GaugeDialPlotOptions dial) {
		setAttr(DAttrs.dial, dial);
	}

	/**
	 * Sets to enable the drilldown.
	 */
	public void setDrilldown(boolean enable) {
		setDrilldown(String.valueOf(enable));
	}

	/**
	 * Sets the id of the drilldown data.
	 */
	public void setDrilldown(String seriesId) {
		_valuesOnly = false;
		setAttr(DAttrs.drilldown, seriesId);
	}

	/**
	 * Returns the id of drilldown
	 * <p>
	 * Default: null
	 */
	public String getDrilldown() {
		return getAttr(DAttrs.drilldown, null).asString();
	}


	/**
	 * Returns individual color for the point. By default the color is pulled
	 * from the global <code>colors</code> array.
	 * <p>
	 * Default: null.
	 */
	public Color getColor() {
		return (Color) getAttr(DAttrs.color, null).asValue();
	}

	/**
	 * Sets individual color for the point. By default the color is pulled from
	 * the global <code>colors</code> array.
	 */
	public void setColor(Color color) {
		_valuesOnly = false;
		setAttr(DAttrs.color, color);
	}

	/**
	 * Sets individual color for the point. By default the color is pulled from
	 * the global <code>colors</code> array.
	 */
	public void setColor(String color) {
		setColor(new Color(color));
	}

	/**
	 * Sets individual color for the point. By default the color is pulled from
	 * the global <code>colors</code> array.
	 */
	public void setColor(LinearGradient color) {
		setColor(new Color(color));
	}

	/**
	 * Sets individual color for the point. By default the color is pulled from
	 * the global <code>colors</code> array.
	 */
	public void setColor(RadialGradient color) {
		setColor(new Color(color));
	}

	/**
	 * Returns the specific color index to use for the point, so its graphic representations are given the class
	 * name highcharts-color-{n}.
	 * <p>
	 * Default: null
	 * @since 7.2.2.0
	 */
	public Number getColorIndex() {
		return getAttr(DAttrs.colorIndex, null).asNumber();
	}

	/**
	 * Sets the specific color index to use for the point, so its graphic representations are given the class
	 * name highcharts-color-{n}.
	 * @since 7.2.2.0
	 */
	public void setColorIndex(Number colorIndex) {
		setAttr(DAttrs.colorIndex, colorIndex, NOT_NULL_VALUE);
	}

	/**
	 * Returns the node that the link runs from.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public String getFrom() {
		return getAttr(DAttrs.from, null).asString();
	}

	/**
	 * Sets the node that the link runs from.
	 * @since 7.2.1.0
	 */
	public void setFrom(String from) {
		_valuesOnly = false;
		setAttr(DAttrs.from, from);
	}

	/**
	 * Returns an id for the point.
	 * <p>
	 * Default: null.
	 */
	public String getId() {
		return getAttr(DAttrs.id, null).asString();
	}

	/**
	 * Sets an id for the point.
	 */
	public void setId(String id) {
		_valuesOnly = false;
		setAttr(DAttrs.id, id);
	}

	/**
	 * Returns the inner radius for points in a solid gauge.
	 * Can be given as a number (pixels) or percentage string.
	 * <p>
	 * Default: 60
	 * @since 7.2.1.0
	 */
	public Object getInnerRadius() {
		return getAttr(DAttrs.innerRadius, 60).asValue();
	}

	/**
	 * Sets the inner radius for points in a solid gauge.
	 * Can be given as a number (pixels).
	 * @since 7.2.1.0
	 */
	public void setInnerRadius(Number innerRadius) {
		_valuesOnly = false;
		setAttr(DAttrs.innerRadius, innerRadius);
	}


	/**
	 * Sets the inner radius for points in a solid gauge.
	 * Can be given as a percentage string.
	 * @since 7.2.1.0
	 */
	public void setInnerRadius(String innerRadius) {
		_valuesOnly = false;
		setAttr(DAttrs.innerRadius, innerRadius);
	}

	/**
	 * Returns waterfall series only. Whether to acts as a summary column for
	 * the values added or subtracted since the last intermediate sum.
	 * <p>
	 * Default: false.
	 */
	public boolean isIntermediateSum() {
		return getAttr(DAttrs.isIntermediateSum, false).asBoolean();
	}

	/**
	 * Sets waterfall series only. Whether to acts as a summary column for the
	 * values added or subtracted since the last intermediate sum.
	 */
	public void setIntermediateSum(boolean sum) {
		_valuesOnly = false;
		setAttr(DAttrs.isIntermediateSum, sum);
	}

	/**
	 * Returns the label.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public String getLabel() {
		return getAttr(DAttrs.label, null).asString();
	}

	/**
	 * Sets the label.
	 * @since 7.2.1.0
	 */
	public void setLabel(String label) {
		_valuesOnly = false;
		setAttr(DAttrs.label, label);
	}

	/**
	 * Returns the rank for this point's data label in case of collision.
	 * If two data labels are about to overlap, only the one with the highest labelrank will be drawn.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Number getLabelrank() {
		return getAttr(DAttrs.labelrank, null).asNumber();
	}

	/**
	 * Sets the rank for this point's data label in case of collision.
	 * If two data labels are about to overlap, only the one with the highest labelrank will be drawn.
	 * @since 7.2.1.0
	 */
	public void setLabelrank(Number labelrank) {
		_valuesOnly = false;
		setAttr(DAttrs.labelrank, labelrank);
	}

	/**
	 * Returns the length of the vector. The rendered length will relate to the vectorLength setting.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Number getLength() {
		return getAttr(DAttrs.length, null).asNumber();
	}

	/**
	 * Sets the length of the vector. The rendered length will relate to the vectorLength setting.
	 * @since 7.2.1.0
	 */
	public void setLength(Number length) {
		_valuesOnly = false;
		setAttr(DAttrs.length, length);
	}

	/**
	 * Returns waterfall series only. Whether to display the total sum across
	 * the entire series.
	 * <p>
	 * Default: false.
	 */
	public boolean isSum() {
		return getAttr(DAttrs.isSum, false).asBoolean();
	}

	/**
	 * Sets waterfall series only. Whether to display the total sum across the
	 * entire series.
	 */
	public void setSum(boolean sum) {
		_valuesOnly = false;
		setAttr(DAttrs.isSum, sum);
	}

	/**
	 * Returns pies only. The sequential index of the pie slice in the legend.
	 * <p>
	 * Default: 0.
	 */
	public int getLegendIndex() {
		return getAttr(DAttrs.legendIndex, 0).asInt();
	}

	/**
	 * Sets pies only. The sequential index of the pie slice in the legend.
	 */
	public void setLegendIndex(int legendIndex) {
		_valuesOnly = false;
		setAttr(DAttrs.legendIndex, legendIndex);
	}

	/**
	 * Returns the data marker
	 * <p>
	 * Default: null.
	 */
	public Marker getMarker() {
		Marker marker = (Marker) getAttr(DAttrs.marker);
		if (marker == null) {
			_valuesOnly = false;
			marker = new Marker();
			setAttr(DAttrs.marker, marker);
		}
		return marker;
	}

	/**
	 * Sets the data marker
	 */
	public void setMarker(Marker marker) {
		_valuesOnly = false;
		setAttr(DAttrs.marker, marker);
	}

	/**
	 * Returns the name of the point as shown in the legend, tooltip, dataLabel
	 * etc.
	 * <p>
	 * Default: null.
	 */
	public String getName() {
		return getAttr(DAttrs.name, null).asString();
	}

	/**
	 * Sets the name of the point as shown in the legend, tooltip, dataLabel
	 * etc.
	 */
	public void setName(String name) {
		_valuesOnly = false;
		setAttr(DAttrs.name, name);
	}

	/**
	 * Returns whether the link goes out of the system.
	 * <p>
	 * Default: false
	 * @since 7.2.1.0
	 */
	public boolean isOutgoing() {
		return getAttr(DAttrs.outgoing, false).asBoolean();
	}

	/**
	 * Set whether the link goes out of the system.
	 * @since 7.2.1.0
	 */
	public void setOutgoing(boolean outgoing) {
		_valuesOnly = false;
		setAttr(DAttrs.outgoing, outgoing);
	}

	/**
	 * Returns a partial fill for each point, if any. Otherwise, create a new one.
	 * Typically used to visualize how much of a task is performed.
	 * The partial fill object can be set either on series or point level.
	 * @since 7.2.1.0
	 */
	public XRangePartialFill getPartialFill() {
		XRangePartialFill partialFill = (XRangePartialFill) getAttr(DAttrs.partialFill);
		if (partialFill == null) {
			partialFill = new XRangePartialFill();
			setPartialFill(partialFill);
		}
		return partialFill;
	}

	/**
	 * Sets a partial fill for each point.
	 * Typically used to visualize how much of a task is performed.
	 * The partial fill object can be set either on series or point level.
	 * @since 7.2.1.0
	 */
	public void setPartialFill(XRangePartialFill partialFill) {
		_valuesOnly = false;
		setAttr(DAttrs.partialFill, partialFill);
	}

	/**
	 * Sets a partial fill with an amount for each point.
	 * Typically used to visualize how much of a task is performed.
	 * The partial fill object can be set either on series or point level.
	 * @since 7.2.1.0
	 */
	public void setPartialFill(Number amount) {
		setPartialFill(new XRangePartialFill(amount));
	}

	/**
	 * Returns the outer radius for points in a solid gauge.
	 * Can be given as a number (pixels) or percentage string.
	 * <p>
	 * Default: 100
	 * @since 7.2.1.0
	 */
	public Object getRadius() {
		return getAttr(DAttrs.radius, 100).asValue();
	}

	/**
	 * Sets the outer radius for points in a solid gauge.
	 * Can be given as a number (pixels).
	 * @since 7.2.1.0
	 */
	public void setRadius(Number radius) {
		_valuesOnly = false;
		setAttr(DAttrs.radius, radius);
	}

	/**
	 * Sets the outer radius for points in a solid gauge.
	 * Can be given as a percentage string.
	 * @since 7.2.1.0
	 */
	public void setRadius(String radius) {
		_valuesOnly = false;
		setAttr(DAttrs.radius, radius);
	}
	
	/**
	 * Returns whether the point is selected or not.
	 * <p>
	 * Default: false.
	 */
	public boolean isSelected() {
		_valuesOnly = false;
		return getAttr(Attrs.selected, false).asBoolean();
	}

	/**
	 * Sets whether the point is selected or not.
	 */
	public void setSelected(boolean selected) {
		select(selected, false);
	}

	/**
	 * Sets whether the point is selected or not.
	 */
	public void setSelectedDirectly(boolean selected) {
		_valuesOnly = false;
		setAttr(Attrs.selected, selected);
		setSlicedDirectly(selected);
	}

	/**
	 * Returns the set or sets the options will be applied to, if any.
	 * Otherwise, create a new one.
	 * If a single entry is defined, then it will create a new set.
	 * If more than one entry is defined, then it will define the overlap between the sets in the array.
	 * @since 7.2.1.0
	 */
	public Set<String> getSets() {
		Set<String> sets = (Set<String>) getAttr(DAttrs.sets);
		if (sets == null) {
			sets = new HashSet<String>();
			setSets(sets);
		}
		return sets;
	}

	/**
	 * Sets the set or sets the options will be applied to.
	 * If a single entry is defined, then it will create a new set.
	 * If more than one entry is defined, then it will define the overlap between the sets in the array.
	 * @since 7.2.1.0
	 */
	public void setSets(Set<String> sets) {
		_valuesOnly = false;
		setAttr(DAttrs.sets, sets);
	}

	/**
	 * Sets the set or sets the options will be applied to.
	 * If a single entry is defined, then it will create a new set.
	 * If more than one entry is defined, then it will define the overlap between the sets in the array.
	 * @since 7.2.1.0
	 */
	public void setSets(String... sets) {
		setSets(new HashSet<String>(Arrays.asList(sets)));
	}

	/**
	 * Returns pie series only. Whether to display a slice offset from the
	 * center.
	 * <p>
	 * Default: false.
	 */
	public boolean isSliced() {
		_valuesOnly = false;
		return getAttr(Attrs.sliced, false).asBoolean();
	}

	/**
	 * Sets pie series only. Whether to display a slice offset from the center.
	 */
	public void setSliced(boolean sliced) {
		slice(sliced);
	}
	
	/**
	 * Sets pie series only. Whether to display a slice offset from the center.
	 */
	public void setSlicedDirectly(boolean sliced) {
		_valuesOnly = false;
		setAttr(Attrs.sliced, sliced);
	}

	/**
	 * Returns the target value of a point.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Number getTarget() {
		return getAttr(DAttrs.target, null).asNumber();
	}

	/**
	 * Sets the target value of a point.
	 * @since 7.2.1.0
	 */
	public void setTarget(Number target) {
		_valuesOnly = false;
		setAttr(DAttrs.target, target);
	}

	/**
	 * Returns individual target options for each point, if any.
	 * Otherwise, create a new one.
	 * @since 7.2.1.0
	 */
	public TargetOptions getTargetOptions() {
		TargetOptions targetOptions = (TargetOptions) getAttr(DAttrs.targetOptions);
		if (targetOptions == null) {
			targetOptions = new TargetOptions();
			setTargetOptions(targetOptions);
		}
		return targetOptions;
	}

	/**
	 * Sets individual target options for each point.
	 * @since 7.2.1.0
	 */
	public void setTargetOptions(TargetOptions targetOptions) {
		_valuesOnly = false;
		setAttr(DAttrs.targetOptions, targetOptions);
	}

	/**
	 * Returns the node that the link runs to.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public String getTo() {
		return getAttr(DAttrs.to, null).asString();
	}

	/**
	 * Sets the node that the link runs to.
	 * @since 7.2.1.0
	 */
	public void setTo(String to) {
		_valuesOnly = false;
		setAttr(DAttrs.to, to);
	}

	/**
	 * Returns the weight of the link.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Number getWeight() {
		return getAttr(DAttrs.weight, null).asNumber();
	}

	/**
	 * Sets the weight of the link.
	 * @since 7.2.1.0
	 */
	public void setWeight(Number weight) {
		_valuesOnly = false;
		setAttr(DAttrs.weight, weight);
	}

	/**
	 * Returns the width of the point.
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public Number getWidth() {
		return getAttr(DAttrs.width, null).asNumber();
	}

	/**
	 * Sets the width of the point.
	 * @since 7.2.1.0
	 */
	public void setWidth(Number width) {
		_valuesOnly = false;
		setAttr(DAttrs.width, width);
	}

	/**
	 * Returns pixel value specifying a fixed width for each column or bar. When null, the width is calculated
	 * from the pointPadding and groupPadding.
	 * <p>
	 * Default: null
	 * @since 7.2.2.0
	 */
	public Number getPointWidth() {
		return getAttr(DAttrs.pointWidth, null).asNumber();
	}

	/**
	 * Sets pixel value specifying a fixed width for each column or bar. When null, the width is calculated
	 * from the pointPadding and groupPadding.
	 * @since 7.2.2.0
	 */
	public void setPointWidth(Number pointWidth) {
		_valuesOnly = false;
		setAttr(DAttrs.pointWidth, pointWidth);
	}

	/**
	 * Returns the x value of the point.
	 * <p>
	 * Default: null.
	 */
	public Number getX() {
		return (Number) getAttr(DAttrs.x, null).asValue();
	}

	/**
	 * Sets the x value of the point.
	 */
	public void setX(Number x) {
		setAttr(DAttrs.x, x, NOT_NULL_VALUE);
	}

	/**
	 * Returns the x2 value of the point.
	 * <p>
	 * Default: null.
	 * @since 7.2.1.0
	 */
	public Number getX2() {
		return (Number) getAttr(DAttrs.x2, null).asValue();
	}

	/**
	 * Sets the x2 value of the point.
	 * @since 7.2.1.0
	 */
	public void setX2(Number x2) {
		_valuesOnly = false;
		setAttr(DAttrs.x2, x2, NOT_NULL_VALUE);
	}

	/**
	 * Returns the y value of the point.
	 * <p>
	 * Default: null.
	 */
	public Number getY() {
		return (Number) getAttr(DAttrs.y, null).asValue();
	}

	/**
	 * Sets the y value of the point.
	 */
	public void setY(Number y) {
		setAttr(DAttrs.y, y, NOT_NULL_VALUE);
	}

	/**
	 * Returns the z value of the point.
	 * <p>
	 * Default: null.
	 * @since 7.2.1.0
	 */
	public Number getZ() {
		return (Number) getAttr(DAttrs.z, null).asValue();
	}

	/**
	 * Sets the z value of the point.
	 * @since 7.2.1.0
	 */
	public void setZ(Number z) {
		setAttr(DAttrs.z, z, NOT_NULL_VALUE);
	}

	/**
	 * Removes the point from the series
	 */
	public void remove() {
		fireEvent(EventType.DESTROYED, "point", this, new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction("remove");
			}

		});
		clearOptonDataListener();
		_series = null;
	}

	/**
	 * Toggle select or unselect the point.
	 */
	public void select() {
		select(!isSelected(), false);
	}
	
	/**
	 * Select or unselect the point.
	 * 
	 * @param select when true, the point is selected. When false, the point is unselected.
	 * @param accumulate when true, the selection is added to other selected points.
	 * When false, other selected points are deselected. Selected points are accumulated on
	 * Control, Shift or Cmd clicking the point.
	 */
	public void select(final boolean select, final boolean accumulate) {
		setSelectedDirectly(select);
		fireEvent(EventType.SELECTED, "accumulate", accumulate, new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction("select", select, accumulate, true);
			}
			
		});
	}

	/**
	 * Pie series only. Slice out or set back in a pie chart slice.
	 */
	public void slice() {
		slice(!isSliced());
	}
	
	/**
	 * Pie series only. Slice out or set back in a pie chart slice.
	 */
	public void slice(final boolean sliced) {
		fireEvent(EventType.CHANGED, "slice", this, new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction("slice", sliced);
			}

		});
		setSlicedDirectly(sliced);
	}

	/**
	 * Updates with the given values, x, low, and high.
	 * 
	 * @param x
	 * @param low
	 * @param high
	 */
	public void update(final Number x, final Number low, final Number high) {
		setX(x);
		setLow(low);
		setHigh(high);
		OptionDataEvent evt = new OptionDataEvent(this, EventType.CHANGED,
				DAttrs.x.toString(), x, DAttrs.low.toString(), low,
				DAttrs.high.toString(), high);
		evt.addJSFunctionCall(new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction(
						"update",
						new JavaScriptValue(JSONArray
								.toJSONString(new Number[] { x, low, high })));
			}
		});
		fireEvent(evt);
	}

	/**
	 * Updates with the given values, x, and y.
	 * 
	 * @param x
	 * @param y
	 */
	public void update(final Number x, final Number y) {
		setX(x);
		setY(y);
		OptionDataEvent evt = new OptionDataEvent(this, EventType.CHANGED,
				DAttrs.x.toString(), x, DAttrs.y.toString(), y);
		evt.addJSFunctionCall(new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction(
						"update",
						new JavaScriptValue(JSONArray
								.toJSONString(new Number[] { x, y })));
			}
		});
		fireEvent(evt);
	}

	/**
	 * Updates with the given new value.
	 */
	public void update(final Number value) {
		setY(value);
		fireEvent(EventType.CHANGED, DAttrs.y, value, new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction("update", value);
			}
		});
	}

	/**
	 * Updates with the given values, from and to.
	 * @since 7.2.1.0
	 */
	public void update(final String from, final String to) {
		setFrom(from);
		setTo(to);
		OptionDataEvent evt = new OptionDataEvent(this, EventType.CHANGED,
				DAttrs.from.toString(), from, DAttrs.to.toString(), to);
		final JSONObject updateData = new JSONObject();
		updateData.put("from", from);
		updateData.put("to", to);
		evt.addJSFunctionCall(new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction("update", new JavaScriptValue(updateData.toJSONString()));
			}
		});
		fireEvent(evt);
	}

	/**
	 * Updates with the given values, from, to and weight.
	 * @since 7.2.1.0
	 */
	public void update(String from, String to, Number weight) {
		setFrom(from);
		setTo(to);
		setWeight(weight);
		OptionDataEvent evt = new OptionDataEvent(this, EventType.CHANGED,
				DAttrs.from.toString(), from,
				DAttrs.to.toString(), to,
				DAttrs.weight.toString(), weight);
		final JSONObject updateData = new JSONObject();
		updateData.put("from", from);
		updateData.put("to", to);
		updateData.put("weight", weight);
		evt.addJSFunctionCall(new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction("update", new JavaScriptValue(updateData.toJSONString()));
			}
		});
		fireEvent(evt);
	}

	/**
	 * Updates with the given values, name and weight.
	 * @since 7.2.1.0
	 */
	public void update(String name, Number weight) {
		setName(name);
		setWeight(weight);
		OptionDataEvent evt = new OptionDataEvent(this, EventType.CHANGED,
				DAttrs.name.toString(), name,
				DAttrs.weight.toString(), weight);
		final JSONObject updateData = new JSONObject();
		updateData.put("name", name);
		updateData.put("weight", weight);
		evt.addJSFunctionCall(new DeferredCall() {
			public void execute(JSFunction func) {
				func.callFunction("update", new JavaScriptValue(updateData.toJSONString()));
			}
		});
		fireEvent(evt);
	}

	/**
	 * Encodes this object to a JSON string. It is the same as
	 * {@link #toString()}.
	 */
	@SuppressWarnings("unchecked")
	public String toJSONString() {
		if (options.isEmpty()) {
			return "null";
		} else if (options.size() == 1 && getY() != null) {
			return getY().toString();
		} else if (_valuesOnly) {
			// for backward compatibility
			return OptionsList.toJSONString(options.values());
		} else
			return JSONObject.toJSONString(options);
	}
	
	/**
	 * Get the "value" value of this point
	 * @return value or null if not set
	 * @since 2.1.0
	 */
	public Number getValue() {
		return getAttr(DAttrs.value, null).asNumber();
	}
	/**
	 * Set the "value" value. As some plots use value instead of y.
	 * @param number
	 * @since 2.1.0
	 */
	public void setValue(Number number) {
		setAttr(DAttrs.value, number, null);
	}
	
	/**
	 * Get the parent point of this point. Only valid for tree maps.
	 * @return parent id or null if not set
	 * @since 2.1.0
	 */
	public String getParent() {
		return getAttr(DAttrs.parent, null).asString();
	}
	/**
	 * Set the parent point of this point. Only valid for tree maps.
	 * @param parent id of parent point
	 * @since 2.1.0
	 */
	public void setParent(String parent) {
		_valuesOnly = false;
		setAttr(DAttrs.parent, parent, null);
	}
	
	/**
	 * Get the color value of this point. Only valid for plots with color axis.
	 * @return the color value or null if not set
	 * @since 2.1.0
	 */
	public Number getColorValue() {
		return getAttr(DAttrs.colorValue, null).asNumber();
	}
	/**
	 * Set the color value for this point. Only valid for plots with color axis.
	 * @param colorValue
	 * @since 2.1.0
	 */
	public void setColorValue(Number colorValue) {
		_valuesOnly = false;
		setAttr(DAttrs.colorValue, colorValue, null);
	}
	
	/**
	 * Get the Q1 (first quartile) value of this point. Only valid for box plot.
	 * @return the Q1 value or null if not set
	 * @since 2.1.0
	 */
	public Number getQ1() {
		return getAttr(DAttrs.q1, null).asNumber();
	}
	/**
	 * Set the Q1 (first quartile) value for this point. Only valid for box plot.
	 * @param q1 the first quartile value or null if not set
	 * @since 2.1.0
	 */
	public void setQ1(Number q1) {
		setAttr(DAttrs.q1, q1, null);
	}
	
	/**
	 * Get the median value of this point. Only valid for box plot.
	 * @return the median value or null if not set
	 * @since 2.1.0
	 */
	public Number getMedian() {
		return getAttr(DAttrs.median, null).asNumber();
	}
	/**
	 * Set the median value for this point. Only valid for box plot.
	 * @param median the median value
	 * @since 2.1.0
	 */
	public void setMedian(Number median) {
		setAttr(DAttrs.median, median, null);
	}
	
	/**
	 * Get the Q3 (third quartile) of this point. Only valid for box plot.
	 * @return the Q3 value or null if not set
	 * @since 2.1.0
	 */
	public Number getQ3() {
		return getAttr(DAttrs.q3, null).asNumber();
	}
	/**
	 * Set the Q3 (third quartile) value for this point. Only valid for box plot.
	 * @param q3 the third quartile value
	 * @since 2.1.0
	 */
	public void setQ3(Number q3) {
		setAttr(DAttrs.q3, q3, null);
	}

	/**
	 * Represents an array like point without any key.
	 * @since 10.2.1.0
	 */
	public static class ArrayPoint<T> extends Point {
		private List<T> data;

		public ArrayPoint(List<T> data) {
			this.data = data;
		}
		public ArrayPoint(T... data) {
			this.data = Arrays.asList(data);
		}
		public ArrayPoint(Number... data) {
			this.data = Generics.cast(Arrays.asList(data));
		}
		public String toJSONString() {
			if (data != null)
				return JSONValue.toJSONString(data);
			return super.toJSONString();
		}
	}
}