/* PointGrouping.java

        Purpose:
                
        Description:
                
        History:
                Fri Sep 15 14:41:19 CST 2023, Created by jamson

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Options for grouping data points together when sonifying.
 * This allows for the visual presentation to contain more points than what is being played.
 * If not enabled, all visible / uncropped points are played.
 * @author Jamson Chan
 * @since 11.2.0.0
 */
public class PointGrouping extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		algorithm,
		enabled,
		groupTimespan,
		prop
	}

	/**
	 * Returns The grouping algorithm, deciding which points to keep when grouping a set of points together.
	 * By default "minmax" is used, which keeps both the minimum and maximum points.
	 * <p>The other algorithms will either keep the first point in the group (time wise),
	 * last point, middle point, or both the first and the last point.
	 * <p>The timing of the resulting point(s) is then adjusted to play evenly,
	 * regardless of its original position within the group.
	 * <p>Default: {@code "minmax"}
	 */
	public String getAlgorithm() {
		return getAttr(Attrs.algorithm, "minmax").asString();
	}

	/**
	 * Sets The grouping algorithm, deciding which points to keep when grouping a set of points together.
	 * By default "minmax" is used, which keeps both the minimum and maximum points.
	 * <p>The other algorithms will either keep the first point in the group (time wise),
	 * last point, middle point, or both the first and the last point.
	 * <p>The timing of the resulting point(s) is then adjusted to play evenly,
	 * regardless of its original position within the group.
	 */
	public void setAlgorithm(String algorithm) {
		setAttr(Attrs.algorithm, algorithm, "minmax");
	}

	/**
	 * Returns whether or not to group points
	 * <p>Default: {@code true}
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, true).asBoolean();
	}

	/**
	 * Sets whether or not to group points
	 */
	public void setEnabled(Boolean enabled) {
		setAttr(Attrs.enabled, enabled, true);
	}

	/**
	 * Returns the size of each group in milliseconds. Audio events closer than this are grouped together.
	 * <p>Default: {@code 15}
	 */
	public Number getGroupTimespan() {
		return getAttr(Attrs.groupTimespan, 15).asNumber();
	}

	/**
	 * Sets the size of each group in milliseconds. Audio events closer than this are grouped together.
	 */
	public void setGroupTimespan(Number groupTimespan) {
		setAttr(Attrs.groupTimespan, groupTimespan, 15);
	}

	/**
	 * Returns the data property for each point to compare when deciding which points to keep in the group.
	 * <p>By default it is "y", which means that if the "minmax" algorithm is used,
	 * the two points the group with the lowest and highest y value will be kept, and the others not played.
	 * <p>Default: {@code "y"}
	 */
	public String getProp() {
		return getAttr(Attrs.prop, "y").asString();
	}

	/**
	 * Sets the data property for each point to compare when deciding which points to keep in the group.
	 * <p>By default it is "y", which means that if the "minmax" algorithm is used,
	 * the two points the group with the lowest and highest y value will be kept, and the others not played.
	 */
	public void setProp(String prop) {
		setAttr(Attrs.prop, prop, "y");
	}

}
