/* SeriesNavigation.java

	Purpose:
		
	Description:
		
	History:
		12:32 PM 2022/9/19, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An options for the keyboard navigation of data points and series.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class SeriesNavigation extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		mode,
		pointNavigationEnabledThreshold,
		rememberPointFocus,
		skipNullPoints
	}

	/**
	 * Returns the keyboard navigation mode for the chart. Can be {@code "normal"} or
	 * {@code "serialize"}.
	 * <p>Default: {@code "normal"}
	 */
	public String getMode() {
		return getAttr(Attrs.mode, "normal").asString();
	}

	/**
	 * Sets the keyboard navigation mode for the chart. Can be {@code "normal"} or
	 * {@code "serialize"}. In normal mode, left/right arrow keys move between points
	 * in a series, while up/down arrow keys move between series.
	 * Up/down navigation acts intelligently to figure out which series makes sense
	 * to move to from any given point.
	 *
	 * <p>
	 * In {@code "serialize"} mode, points are instead navigated as a single list.
	 * Left/right behaves as in {@code "normal"} mode. Up/down arrow keys will behave
	 * like left/right. This can be useful for unifying navigation behavior with/without
	 * screen readers enabled.
	 * <p>Default: {@code "normal"}
	 */
	public void setMode(String mode) {
		if (!("normal".equals(mode) || "serialize".equals(mode))) {
			throw new IllegalArgumentException("Unknown mode: [" + mode + "]");
		}
		setAttr(Attrs.mode, mode);
	}

	/**
	 * Returns whether to enable/disable when a series contains more points than this,
	 * we no longer allow keyboard navigation for it.
	 * <p>Default: {@code false}
	 */
	public boolean isPointNavigationEnabledThreshold() {
		return getAttr(Attrs.pointNavigationEnabledThreshold,
				false).asBoolean();
	}

	/**
	 * Sets whether to enable/disable when a series contains more points than this,
	 * we no longer allow keyboard navigation for it.
	 * <p>Default: {@code false}
	 */
	public void setPointNavigationEnabledThreshold(
			boolean pointNavigationEnabledThreshold) {
		setAttr(Attrs.pointNavigationEnabledThreshold,
				pointNavigationEnabledThreshold);
	}

	/**
	 * Returns whether to remember which point was focused even after navigating
	 * away from the series, so that when navigating back to the series you start
	 * at the last focused point.
	 * <p>Default: {@code false}
	 */
	public boolean isRememberPointFocus() {
		return getAttr(Attrs.rememberPointFocus, false).asBoolean();
	}

	/**
	 * Sets whether to remember which point was focused even after navigating
	 * away from the series, so that when navigating back to the series you start
	 * at the last focused point.
	 * <p>Default: {@code false}
	 */
	public void setRememberPointFocus(boolean rememberPointFocus) {
		setAttr(Attrs.rememberPointFocus, rememberPointFocus);
	}

	/**
	 * Returns whether to skip null points when navigating through points with the keyboard
	 * <p>Default: {@code true}
	 */
	public boolean isSkipNullPoints() {
		return getAttr(Attrs.skipNullPoints, true).asBoolean();
	}

	/**
	 * Sets whether to skip null points when navigating through points with the keyboard
	 * <p>Default: {@code true}
	 */
	public void setSkipNullPoints(boolean skipNullPoints) {
		setAttr(Attrs.skipNullPoints, skipNullPoints);
	}
}