/** StackLabels.java.

	Purpose:
		
	Description:
		
	History:
		9:23:13 AM Jan 13, 2014, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.
 */

package org.zkoss.chart;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.lang.Generics;

/**
 * The stack labels show the total value for each bar in a stacked column or bar
 * chart. The label will be placed on top of positive columns and below negative
 * columns. In case of an inverted column chart or a bar chart the label is
 * placed to the right of positive bars and to the left of negative bars.
 * 
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author jumperchen
 * @author RaymondChao
 */
public class StackLabels extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		align,
		enabled,
		format,
		formatter,
		rotation,
		style,
		textAlign,
		useHTML,
		verticalAlign,
		x,
		y
	}

	/**
	 * Returns the horizontal alignment of the stack total label. Can be one of
	 * <code>"left"</code>, <code>"center"</code> or <code>"right"</code>. The
	 * default value is calculated at runtime and depends on orientation and
	 * whether the stack is positive or negative.
	 * <p>
	 * Default: null.
	 */
	public String getAlign() {
		return getAttr(Attrs.align, null).asString();
	}

	/**
	 * Sets the horizontal alignment of the stack total label. Can be one of
	 * <code>"left"</code>, <code>"center"</code> or <code>"right"</code>. The
	 * default value is calculated at runtime and depends on orientation and
	 * whether the stack is positive or negative.
	 */
	public void setAlign(String align) {
		setAttr(Attrs.align, align);
	}

	/**
	 * Returns whether to enable or disable the stack total labels.
	 * <p>
	 * Default: false.
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, false).asBoolean();
	}

	/**
	 * Sets whether to enable or disable the stack total labels.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}

	/**
	 * Returns a <a href="http://docs.highcharts.com/#formatting">format
	 * string</a> for the data label. Available variables are the same as for
	 * <code>formatter</code>.
	 * <p>
	 * Default: "{total}".
	 */
	public String getFormat() {
		return getAttr(Attrs.format, "{total}").asString();
	}

	/**
	 * Sets a <a href="http://docs.highcharts.com/#formatting">format string</a>
	 * for the data label. Available variables are the same as for
	 * <code>formatter</code>.
	 * <p>
	 * Default: "{total}".
	 */
	public void setFormat(String format) {
		setAttr(Attrs.format, format, "{total}");
	}

	/**
	 * Returns rotation of the labels in degrees.
	 * <p>
	 * Default: 0.
	 */
	public Number getRotation() {
		return getAttr(Attrs.rotation, 0).asNumber();
	}

	/**
	 * Sets rotation of the labels in degrees.
	 */
	public void setRotation(Number rotation) {
		setAttr(Attrs.rotation, rotation, 0);
	}

	/**
	 * Returns CSS styles for the label.
	 * <p>
	 * Defaults to:
	 * 
	 * <pre>
	 * color: #666; font-size: 11px; line-height: 14px;
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getStyle() {
		if (!containsKey(Attrs.style)) {
			setStyle("color: #666; font-size: 11px; line-height: 14px;");
		}
		return Generics.cast(getAttr(Attrs.style));
	}

	/**
	 * Sets CSS styles for the label.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 */
	public void setStyle(String style) {
		setStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the label.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style, NOT_NULL_VALUE);
	}

	/**
	 * Returns the text alignment for the label. While <code>align</code>
	 * determines where the texts anchor point is placed with regards to the
	 * stack, <code>textAlign</code> determines how the text is aligned against
	 * its anchor point. Possible values are <code>"left"</code>,
	 * <code>"center"</code> and <code>"right"</code>. The default value is
	 * calculated at runtime and depends on orientation and whether the stack is
	 * positive or negative.
	 * <p>
	 * Default: null.
	 */
	public String getTextAlign() {
		return getAttr(Attrs.textAlign, null).asString();
	}

	/**
	 * Sets the text alignment for the label. While <code>align</code>
	 * determines where the texts anchor point is placed with regards to the
	 * stack, <code>textAlign</code> determines how the text is aligned against
	 * its anchor point. Possible values are <code>"left"</code>,
	 * <code>"center"</code> and <code>"right"</code>. The default value is
	 * calculated at runtime and depends on orientation and whether the stack is
	 * positive or negative.
	 * <p>
	 * Default: null.
	 */
	public void setTextAlign(String textAlign) {
		if (!("left".equals(textAlign) || "center".equals(textAlign) || "right"
				.equals(textAlign)))
			throw new IllegalArgumentException("Unsupported textAlign: ["
					+ textAlign + "]");
		setAttr(Attrs.textAlign, textAlign);
	}

	/**
	 * Returns whether to <a
	 * href="http://docs.highcharts.com/#formatting$html">use HTML</a> to render
	 * the labels.
	 * <p>
	 * Default: false.
	 */
	public boolean isUseHTML() {
		return getAttr(Attrs.useHTML, false).asBoolean();
	}

	/**
	 * Sets whether to <a href="http://docs.highcharts.com/#formatting$html">use
	 * HTML</a> to render the labels.
	 */
	public void setUseHTML(boolean useHTML) {
		setAttr(Attrs.useHTML, useHTML);
	}

	/**
	 * Returns defines the vertical alignment of the stack total label. Can be
	 * one of <code>"top"</code>, <code>"middle"</code> or <code>"bottom"</code>
	 * . The default value is calculated at runtime and depends on orientation
	 * and whether the stack is positive or negative.
	 * <p>
	 * Default: null.
	 */
	public String getVerticalAlign() {
		return getAttr(Attrs.verticalAlign, null).asString();
	}

	/**
	 * Sets defines the vertical alignment of the stack total label. Can be one
	 * of <code>"top"</code>, <code>"middle"</code> or <code>"bottom"</code>.
	 * The default value is calculated at runtime and depends on orientation and
	 * whether the stack is positive or negative.
	 */
	public void setVerticalAlign(String verticalAlign) {
		if (!("top".equals(verticalAlign) || "middle".equals(verticalAlign) || "bottom"
				.equals(verticalAlign)))
			throw new IllegalArgumentException("Unsupported vertical align: ["
					+ verticalAlign + "]");
		setAttr(Attrs.verticalAlign, verticalAlign);
	}

	/**
	 * Returns the x position offset of the label relative to the left of the
	 * stacked bar. The default value is calculated at runtime and depends on
	 * orientation and whether the stack is positive or negative.
	 * <p>
	 * Default: null.
	 */
	public Number getX() {
		return getAttr(Attrs.x, null).asNumber();
	}

	/**
	 * Sets the x position offset of the label relative to the left of the
	 * stacked bar. The default value is calculated at runtime and depends on
	 * orientation and whether the stack is positive or negative.
	 */
	public void setX(Number x) {
		setAttr(Attrs.x, x);
	}

	/**
	 * Returns the y position offset of the label relative to the tick position
	 * on the axis. The default value is calculated at runtime and depends on
	 * orientation and whether the stack is positive or negative.
	 * <p>
	 * Default: null.
	 */
	public Number getY() {
		return getAttr(Attrs.y, null).asNumber();
	}

	/**
	 * Sets the y position offset of the label relative to the tick position on
	 * the axis. The default value is calculated at runtime and depends on
	 * orientation and whether the stack is positive or negative.
	 */
	public void setY(Number y) {
		setAttr(Attrs.y, y);
	}

}