/** State.java.

	Purpose:
		
	Description:
		
	History:
		12:46:14 PM Feb 26, 2014, Created by jumperchen

Copyright (C) 2014 Potix Corporation. All Rights Reserved.
 */
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * A state options.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author jumperchen
 */
public class State extends Optionable {

	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		animation,
		brightness,
		enabled,
		fillColor,
		halo,
		lineColor,
		lineWidth,
		lineWidthPlus,
		radius,
		radiusPlus,
		marker,
		opacity
	}

	private String _name;
	
	/**
	 * Constructs with a name. Like "hover" or "select"
	 */
	public State(String name) {
		_name = name;
	}
	
	/**
	 * Returns the name of this state. Like "hover" or "select"
	 */
	public String getName() {
		return _name;
	}
	/**
	 * Returns enable or disable visible feedback for selection.
	 * 
	 * @return enabled enable or disable visible feedback for selection.
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, true).asBoolean();
	}

	/**
	 * Sets enable or disable visible feedback for selection. 'enabled enable or
	 * disable visible feedback for selection.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}

	/**
	 * Returns the fill color of the point marker.
	 * <p>
	 * Default: null.
	 */
	public Color getFillColor() {
		return (Color) getAttr(Attrs.fillColor, null).asValue();
	}

	/**
	 * Sets the fill color of the point marker.
	 */
	public void setFillColor(Color color) {
		setAttr(Attrs.fillColor, color);
	}

	/**
	 * Sets the fill color of the point marker.
	 */
	public void setFillColor(String color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets the fill color of the point marker.
	 */
	public void setFillColor(LinearGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets the fill color of the point marker.
	 */
	public void setFillColor(RadialGradient color) {
		setFillColor(new Color(color));
	}
	
	/**
	 * Returns the options for the halo appearing around the hovered point 
	 * in line-type series as well as outside the hovered slice in pie charts.
	 * By default the halo is filled by the current point or series color with 
	 * an opacity of 0.25. The halo can be disabled by setting the halo option to false.
	 * <p>
	 * Default: an instance of {@link Halo}.
	 */
	public Halo getHalo() {
		Halo halo = (Halo) getAttr(Attrs.halo);
		if (halo == null) {
			halo = new Halo();
			setHalo(halo);
		}
		return halo;
	}
	
	/**
	 * Returns whether the halo option is disabled or not.
	 */
	public boolean isHaloEnabled() {
		return getAttr(Attrs.halo, null).asBoolean();
	}
	
	/**
	 * Sets the options for the halo appearing around the hovered point 
	 * in line-type series as well as outside the hovered slice in pie charts.
	 * By default the halo is filled by the current point or series color with 
	 * an opacity of 0.25. The halo can be disabled by setting the halo option to false.
	 */
	public void setHalo(Halo halo) {
		setAttr(Attrs.halo, halo);
	}
	
	/**
	 * Sets whether to disable the halo options or not.
	 */
	public void setHaloAsBoolean(boolean halo) {
		setAttr(Attrs.halo, halo);
	}

	/**
	 * Returns the color of the point marker's outline. When <code>null</code>,
	 * the series' or point's color is used.
	 * <p>
	 * Default: "#000000"
	 */
	public Color getLineColor() {
		if (!containsKey(Attrs.lineColor)) {
			setLineColor("#000000");
		}
		return (Color) getAttr(Attrs.lineColor);
	}

	/**
	 * Sets the color of the point marker's outline. When <code>null</code>, the
	 * series' or point's color is used.
	 */
	public void setLineColor(Color color) {
		setAttr(Attrs.lineColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the color of the point marker's outline. When <code>null</code>, the
	 * series' or point's color is used.
	 */
	public void setLineColor(String color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets the color of the point marker's outline. When <code>null</code>, the
	 * series' or point's color is used.
	 */
	public void setLineColor(LinearGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets the color of the point marker's outline. When <code>null</code>, the
	 * series' or point's color is used.
	 */
	public void setLineColor(RadialGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Returns the width of the point marker's outline.
	 */
	public Number getLineWidth() {
		return getAttr(Attrs.lineWidth, 0).asNumber();
	}

	/**
	 * Sets the width of the point marker's outline.
	 */
	public void setLineWidth(Number lineWidth) {
		setAttr(Attrs.lineWidth, lineWidth, NOT_NULL_VALUE);
	}
	
	/**
	 * Returns the additional line width for a hovered point. Defaults to 1
	 */
	public Number getLineWidthPlus() {
		return getAttr(Attrs.lineWidthPlus, 1).asNumber();
	}

	/**
	 * Sets the additional line width for a hovered point. Defaults to 1
	 */
	public void setLineWidthPlus(Number lineWidthPlus) {
		setAttr(Attrs.lineWidthPlus, lineWidthPlus, 1);
	}

	/**
	 * Returns the radius of the point marker. In hover state, it defaults to
	 * the normal state's radius + 2.
	 */
	public Number getRadius() {
		return getAttr(Attrs.radius, null).asNumber();
	}

	/**
	 * Sets the radius of the point marker. In hover state, it defaults to the
	 * normal state's radius + 2.
	 */
	public void setRadius(Number radius) {
		setAttr(Attrs.radius, radius);
	}
	
	/**
	 * Returns the number of pixels to increase the radius of the hovered point. Defaults to 2.
	 */
	public Number getRadiusPlus() {
		return getAttr(Attrs.radiusPlus, 2).asNumber();
	}

	/**
	 * Sets the number of pixels to increase the radius of the hovered point. Defaults to 2.
	 */
	public void setRadiusPlus(Number radiusPlus) {
		setAttr(Attrs.radiusPlus, radiusPlus, 2);
	}
	
	/**
	 * Sets the brightness number
	 */
	public void setBrightness(Number brightness) {
		setAttr(Attrs.brightness, brightness, NOT_NULL_VALUE);
	}

	/**
	 * Returns the brightness number
	 */
	public Number getBrightness(Number brightness) {
		return getAttr(Attrs.brightness, null).asNumber();
	}

	/**
	 * Returns the marker of this state
	 */
	public Marker getMarker() {
		Marker marker = (Marker) getAttr(Attrs.marker);
		if (marker == null) {
			marker = new Marker();
			setMarker(marker);
		}
		return marker;
	}

	/**
	 * Sets the marker of this state
	 * @param marker
	 */
	public void setMarker(Marker marker) {
		setAttr(Attrs.marker, marker);
	}

	/**
	 * Returns whether to enable or disable the initial animation of this state.
	 * <p>
	 * Default: true.
	 * @since 7.2.1.0
	 */
	public boolean isAnimation() {
		return getAnimation() != Animation.NONE;
	}

	/**
	 * Sets whether to enable or disable the initial animation of this state.
	 * @see #setAnimation(Animation)
	 */
	public void setAnimation(boolean animation) {
		setAnimation(animation ? new Animation() : Animation.NONE);
	}

	/**
	 * Returns whether to enable or disable the initial animation of this state.
	 * <p>
	 * Default: an instance of {@link Animation}.
	 * @since 7.2.1.0
	 */
	public Animation getAnimation() {
		return (Animation) getAttr(Attrs.animation, Animation.class).asValue();
	}

	/**
	 * Sets the initial animation of this state.
	 * @see #setAnimation(boolean)
	 * @since 7.2.1.0
	 */
	public void setAnimation(Animation animation) {
		setAttr(Attrs.animation, animation == null ? Animation.NONE : animation);
	}

	/**
	 * Inactive state only.
	 * Returns the opacity of series elements (dataLabels, line, area).
	 * Set to 1 to disable inactive state.
	 * <p>
	 * Default: 0.2
	 * @since 7.2.1.0
	 */
	public Number getOpacity() {
		return getAttr(Attrs.opacity, 0.2).asNumber();
	}

	/**
	 * Inactive state only.
	 * Sets the opacity of series elements (dataLabels, line, area).
	 * Set to 1 to disable inactive state.
	 * @since 7.2.1.0
	 */
	public void setOpacity(Number opacity) {
		setAttr(Attrs.opacity, opacity, 0.2);
	}
}
