/** XAxis.java.

	Purpose:
		
	Description:
		
	History:
		09:20:17 AM Jan 13, 2014, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.
 */

package org.zkoss.chart;

import java.util.List;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * The X axis or category axis. Normally this is the horizontal axis, though if
 * the chart is inverted this is the vertical axis. In case of multiple axes,
 * the xAxis node is an array of configuration objects.
 * 
 * @author RaymondChao
 */
public class XAxis extends Axis {
	private enum DAttrs implements PlotAttribute, DynamicalAttribute {
		breaks,
		crosshair,
		height,
		left,
		lineColor,
		lineWidth,
		showEmpty,
		top,
		width
	}

	/**
	 * Get the list of breaks defined for this axis.
	 * @return list of breaks
	 * @see Break
	 * @since 2.1.0
	 */
	public List<Break> getBreaks() {
		return Generics.cast(getAttr(DAttrs.breaks, null).asValue());
	}

	/**
	 * Set the list of breaks defined for this axis.
	 * @param breaks
	 * @see Break
	 * @since 2.1.0
	 */
	public void setBreaks(List<Break> breaks) {
		setAttr(DAttrs.breaks, breaks);
	}

	/**
	 * Get the crosshair config object for this axis.
	 * @return Crosshair config object, or create a new one if not set/disabled
	 * @since 2.1.1
	 * @see #setCrosshair(Crosshair)
	 */
	public Crosshair getCrosshair() {
		return Generics.cast(getAttr(DAttrs.crosshair, Crosshair.class).asValue());
	}

	/**
	 * Enable crosshair with default values, or disable crosshair.
	 * @see #setCrosshair(Crosshair)
	 * @since 2.1.1
	 * @param crosshair true to enable, false to disable
	 */
	public void setCrosshair(boolean crosshair) {
		setAttr(DAttrs.crosshair, crosshair);
	}

	/**
	 * Set whether to enable crosshair for this axis or not.
	 * @param crosshair
	 * @since 2.1.1
	 * @see Crosshair
	 */
	public void setCrosshair(Crosshair crosshair) {
		setAttr(DAttrs.crosshair, crosshair);
	}

	/**
	 * Returns the height as the horizontal axis. If it's a number, it is interpreted as pixels.
	 * <p>
	 * Default: undefined
	 * @since 10.2.1.0
	 */
	public String getHeight() {
		return getAttr(DAttrs.height, null).asString();
	}

	/**
	 * Sets the height as the horizontal axis. If it's a number, it is interpreted as pixels.
	 * <p>
	 * Default: undefined
	 * @since 10.2.1.0
	 */
	public void setHeight(String height) {
		setAttr(DAttrs.height, height, null);
	}

	/**
	 * Returns the left position as the horizontal axis. If it's a number,
	 * it is interpreted as pixel position relative to the chart.
	 * <p>
	 * Default: undefined
	 * @since 10.2.1.0
	 */
	public Object getLeft() {
		return getAttr(DAttrs.left, null).asValue();
	}

	/**
	 * Sets the left position as the horizontal axis. If it's a number,
	 * it is interpreted as pixel position relative to the chart.
	 * <p>
	 * Default: undefined
	 * @since 10.2.1.0
	 */
	public void setLeft(String left) {
		setAttr(DAttrs.left, left, null);
	}

	/**
	 * Sets the left position as the horizontal axis. If it's a number,
	 * it is interpreted as pixel position relative to the chart.
	 * <p>
	 * Default: undefined
	 * @since 10.2.1.0
	 */
	public void setLeft(Number left) {
		setAttr(DAttrs.left, left, null);
	}

	/**
	 * Returns the color of the line marking the axis itself.
	 * <p>
	 * Default: "#C0D0E0".
	 */
	public Color getLineColor() {
		if (!containsKey(DAttrs.lineColor)) {
			setLineColor("#C0D0E0");
		}
		return (Color) getAttr(DAttrs.lineColor);
	}

	/**
	 * Sets the color of the line marking the axis itself.
	 */
	public void setLineColor(Color color) {
		setAttr(DAttrs.lineColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the color of the line marking the axis itself.
	 */
	public void setLineColor(String color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets the color of the line marking the axis itself.
	 */
	public void setLineColor(LinearGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets the color of the line marking the axis itself.
	 */
	public void setLineColor(RadialGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Returns the width of the line marking the axis itself.
	 * <p>
	 * Default: 1.
	 */
	public Number getLineWidth() {
		return getAttr(DAttrs.lineWidth, 1).asNumber();
	}

	/**
	 * Sets the width of the line marking the axis itself.
	 */
	public void setLineWidth(Number lineWidth) {
		setAttr(DAttrs.lineWidth, lineWidth, 1);
	}

	/**
	 * Returns whether to show the axis line and title when the axis has no
	 * data.
	 * <p>
	 * Default: true.
	 */
	public boolean isShowEmpty() {
		return getAttr(DAttrs.showEmpty, true).asBoolean();
	}

	/**
	 * Sets whether to show the axis line and title when the axis has no data.
	 */
	public void setShowEmpty(boolean showEmpty) {
		setAttr(DAttrs.showEmpty, showEmpty);
	}

	/**
	 * Returns the top position as the horizontal axis. If it's a number,
	 * it is interpreted as pixel position relative to the chart.
	 * <p>
	 * Default: undefined
	 * @since 10.2.1.0
	 */
	public Object getTop() {
		return getAttr(DAttrs.top, null).asValue();
	}

	/**
	 * Sets the top position as the horizontal axis. If it's a number,
	 * it is interpreted as pixel position relative to the chart.
	 * <p>
	 * Default: undefined
	 * @since 10.2.1.0
	 */
	public void setTop(String top) {
		setAttr(DAttrs.top, top, null);
	}

	/**
	 * Sets the top position as the horizontal axis. If it's a number,
	 * it is interpreted as pixel position relative to the chart.
	 * <p>
	 * Default: undefined
	 * @since 10.2.1.0
	 */
	public void setTop(Number top) {
		setAttr(DAttrs.top, top, null);
	}

	/**
	 * Returns the width as the horizontal axis. If it's a number, it is interpreted as pixels.
	 * <p>
	 * Default: undefined
	 * @since 10.2.1.0
	 */
	public String getWidth() {
		return getAttr(DAttrs.width, null).asString();
	}

	/**
	 * Sets the width as the horizontal axis. If it's a number, it is interpreted as pixels.
	 * <p>
	 * Default: undefined
	 * @since 10.2.1.0
	 */
	public void setWidth(String width) {
		setAttr(DAttrs.width, width, null);
	}
}
