/* BellCurvePlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Wed Nov 20 10:59:12 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.Color;
import org.zkoss.chart.LinearGradient;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.RadialGradient;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An option for bell curve plot
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class BellCurvePlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		fillColor,
		fillOpacity,
		intervals,
		lineColor,
		pointsInInterval,
		negativeFillColor,
		trackByArea
	}

	/**
	 * Returns fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public Color getFillColor() {
		return (Color) getAttr(Attrs.fillColor, null).asValue();
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(Color color) {
		setAttr(Attrs.fillColor, color);
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(String color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(LinearGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(RadialGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Returns fill opacity for the area, arearange, areaspline,  areasplinerange, bellcurve, and streamgraph.
	 * <p>
	 * Default: 0.75.
	 * @since 10.2.1.0
	 */
	public Number getFillOpacity() {
		return getAttr(Attrs.fillOpacity, 0.75).asNumber();
	}

	/**
	 * Sets fill opacity for the area, arearange, areaspline,  areasplinerange, bellcurve, and streamgraph.
	 * @since 10.2.1.0
	 */
	public void setFillOpacity(Number fillOpacity) {
		setAttr(Attrs.fillOpacity, fillOpacity);
	}

	/**
	 * Returns the length of the bell curve.
	 * A unit of the length of the bell curve is standard deviation.
	 * <p>
	 * Default: 3
	 * @since 7.2.1.0
	 */
	public Number getIntervals() {
		return getAttr(Attrs.intervals, 3).asNumber();
	}

	/**
	 * Sets the length of the bell curve.
	 * A unit of the length of the bell curve is standard deviation.
	 * @since 7.2.1.0
	 */
	public void setIntervals(Number intervals) {
		setAttr(Attrs.intervals, intervals, 3);
	}

	/**
	 * Returns a separate color for the graph line. By default the line takes
	 * the <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public Color getLineColor() {
		return (Color) getAttr(Attrs.lineColor, null).asValue();
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(Color color) {
		setAttr(Attrs.lineColor, color);
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(String color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(LinearGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(RadialGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Returns a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public Color getNegativeFillColor() {
		return (Color) getAttr(Attrs.negativeFillColor, null).asValue();
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(Color color) {
		setAttr(Attrs.negativeFillColor, color);
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(String color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(LinearGradient color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(RadialGradient color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Returns how many points should be plotted within 1 interval.
	 * See {@link #getIntervals()}.
	 * <p>
	 * Default: 3
	 * @since 7.2.1.0
	 */
	public Number getPointsInInterval() {
		return getAttr(Attrs.pointsInInterval, 3).asNumber();
	}

	/**
	 * Sets how many points should be plotted within 1 interval.
	 * See {@link #getIntervals()}.
	 * @since 7.2.1.0
	 */
	public void setPointsInInterval(Number pointsInInterval) {
		setAttr(Attrs.pointsInInterval, pointsInInterval, 3);
	}

	/**
	 * Returns whether the whole area or just the line should respond to
	 * mouseover tooltips and other mouse or touch events.
	 * <p>
	 * Default: false.
	 * @since 10.2.1.0
	 */
	public boolean getTrackByArea() {
		return getAttr(Attrs.trackByArea, false).asBoolean();
	}

	/**
	 * Sets whether the whole area or just the line should respond to mouseover
	 * tooltips and other mouse or touch events.
	 * @since 10.2.1.0
	 */
	public void setTrackByArea(boolean trackByArea) {
		setAttr(Attrs.trackByArea, trackByArea);
	}
}
