/* DragDrop.java

	Purpose:
		
	Description:
		
	History:
		Fri Nov 15 15:48:53 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.Color;
import org.zkoss.chart.Optionable;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;

/**
 * The draggable-points module allows points to be moved around or modified in the chart.
 * In addition to the options mentioned under the dragDrop API structure,
 * the module fires three events, <code>point.dragStart</code>, <code>point.drag</code> and <code>point.drop</code>.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class DragDrop extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		draggableX, draggableX1, draggableX2, draggableY, dragHandle, dragMaxX,
		dragMaxY, dragMinX, dragMinY, dragPrecisionX, dragPrecisionY,
		dragSensitivity, groupBy, guideBox, liveRedraw
	}

	/**
	 * Returns if enable dragging in the X dimension.
	 * Note that this is not supported for TreeGrid axes (the default axis type in Gantt charts).
	 * <p>
	 * Default: false
	 */
	public boolean isDraggableX() {
		return getAttr(Attrs.draggableX, false).asBoolean();
	}

	/**
	 * Sets if enable dragging in the X dimension.
	 * Note that this is not supported for TreeGrid axes (the default axis type in Gantt charts).
	 */
	public void setDraggableX(boolean draggableX) {
		setAttr(Attrs.draggableX, draggableX);
	}

	/**
	 * (X-Range) Returns if enable dragging x value individually.
	 * <p>
	 * Default: true
	 */
	public boolean isDraggableX1() {
		return getAttr(Attrs.draggableX1, true).asBoolean();
	}

	/**
	 * (X-Range) Sets if enable dragging x value individually.
	 */
	public void setDraggableX1(boolean draggableX1) {
		setAttr(Attrs.draggableX1, draggableX1);
	}

	/**
	 * (X-Range) Returns if enable dragging x2 value individually.
	 * <p>
	 * Default: true
	 */
	public boolean isDraggableX2() {
		return getAttr(Attrs.draggableX2, true).asBoolean();
	}

	/**
	 * (X-Range) Sets if enable dragging x2 value individually.
	 */
	public void setDraggableX2(boolean draggableX2) {
		setAttr(Attrs.draggableX2, draggableX2);
	}

	/**
	 * Returns if enable dragging in the Y dimension.
	 * Note that this is not supported for TreeGrid axes (the default axis type in Gantt charts).
	 * <p>
	 * Default: false
	 */
	public boolean isDraggableY() {
		return getAttr(Attrs.draggableY, false).asBoolean();
	}

	/**
	 * Sets if enable dragging in the Y dimension.
	 * Note that this is not supported for TreeGrid axes (the default axis type in Gantt charts).
	 */
	public void setDraggableY(boolean draggableY) {
		setAttr(Attrs.draggableY, draggableY);
	}

	/**
	 * Returns the options for the drag handles, if any. Otherwise, create a new one.
	 */
	public DragHandle getDragHandle() {
		DragHandle dragHandle = (DragHandle) getAttr(Attrs.dragHandle);
		if (dragHandle == null) {
			dragHandle = new DragHandle();
			setDragHandle(dragHandle);
		}
		return dragHandle;
	}

	/**
	 * Sets the options for the drag handles.
	 */
	public void setDragHandle(DragHandle dragHandle) {
		setAttr(Attrs.dragHandle, dragHandle);
	}

	/**
	 * Returns the maximum X value the points can be moved to.
	 * <p>
	 * Default: null
	 */
	public Number getDragMaxX() {
		return getAttr(Attrs.dragMaxX, null).asNumber();
	}

	/**
	 * Sets the maximum X value the points can be moved to.
	 */
	public void setDragMaxX(Number dragMaxX) {
		setAttr(Attrs.dragMaxX, dragMaxX);
	}

	/**
	 * Returns the maximum Y value the points can be moved to.
	 * <p>
	 * Default: null
	 */
	public Number getDragMaxY() {
		return getAttr(Attrs.dragMaxY, null).asNumber();
	}

	/**
	 * Sets the maximum Y value the points can be moved to.
	 */
	public void setDragMaxY(Number dragMaxY) {
		setAttr(Attrs.dragMaxY, dragMaxY);
	}

	/**
	 * Returns the minimum X value the points can be moved to.
	 * <p>
	 * Default: null
	 */
	public Number getDragMinX() {
		return getAttr(Attrs.dragMinX, null).asNumber();
	}

	/**
	 * Sets the minimum X value the points can be moved to.
	 */
	public void setDragMinX(Number dragMinX) {
		setAttr(Attrs.dragMinX, dragMinX);
	}

	/**
	 * Returns the minimum Y value the points can be moved to.
	 * <p>
	 * Default: null
	 */
	public Number getDragMinY() {
		return getAttr(Attrs.dragMinY, null).asNumber();
	}

	/**
	 * Sets the minimum Y value the points can be moved to.
	 */
	public void setDragMinY(Number dragMinY) {
		setAttr(Attrs.dragMinY, dragMinY);
	}

	/**
	 * Returns the X precision value to drag to for this series.
	 * Set to 0 to disable. By default this is disabled, except for category axes, where the default is 1.
	 * <p>
	 * Default: 0
	 */
	public Number getDragPrecisionX() {
		return getAttr(Attrs.dragPrecisionX, 0).asNumber();
	}

	/**
	 * Sets the X precision value to drag to for this series.
	 * Set to 0 to disable. By default this is disabled, except for category axes, where the default is 1.
	 */
	public void setDragPrecisionX(Number dragPrecisionX) {
		setAttr(Attrs.dragPrecisionX, dragPrecisionX, 0);
	}

	/**
	 * Returns the Y precision value to drag to for this series.
	 * Set to 0 to disable. By default this is disabled, except for category axes, where the default is 1.
	 * <p>
	 * Default: 0
	 */
	public Number getDragPrecisionY() {
		return getAttr(Attrs.dragPrecisionY, 0).asNumber();
	}

	/**
	 * Sets the Y precision value to drag to for this series.
	 * Set to 0 to disable. By default this is disabled, except for category axes, where the default is 1.
	 */
	public void setDragPrecisionY(Number dragPrecisionY) {
		setAttr(Attrs.dragPrecisionY, dragPrecisionY, 0);
	}

	/**
	 * Returns the amount of pixels to drag the pointer before it counts as a drag operation.
	 * This prevents drag/drop to fire when just clicking or selecting points.
	 * <p>
	 * Default: 2
	 */
	public Number getDragSensitivity() {
		return getAttr(Attrs.dragSensitivity, 2).asNumber();
	}

	/**
	 * Sets the amount of pixels to drag the pointer before it counts as a drag operation.
	 * This prevents drag/drop to fire when just clicking or selecting points.
	 */
	public void setDragSensitivity(Number dragSensitivity) {
		setAttr(Attrs.dragSensitivity, dragSensitivity, 2);
	}

	/**
	 * Returns a property to group the points.
	 * Points with the same property value will be grouped together when moving.
	 * <p>
	 * Default: null
	 */
	public String getGroupBy() {
		return getAttr(Attrs.groupBy, null).asString();
	}

	/**
	 * Sets a property to group the points.
	 * Points with the same property value will be grouped together when moving.
	 */
	public void setGroupBy(String groupBy) {
		setAttr(Attrs.groupBy, groupBy);
	}

	/**
	 * Returns the style options for the guide box, if any. Otherwise, create a new one.
	 * The guide box has one state by default, the default state.
	 */
	public GuideBox getGuideBox() {
		GuideBox guideBox = (GuideBox) getAttr(Attrs.guideBox);
		if (guideBox == null) {
			guideBox = new GuideBox();
			setGuideBox(guideBox);
		}
		return guideBox;
	}

	/**
	 * Sets the style options for the guide box.
	 * The guide box has one state by default, the default state.
	 */
	public void setGuideBox(GuideBox guideBox) {
		setAttr(Attrs.guideBox, guideBox);
	}

	/**
	 * Returns if updates points as they are dragged.
	 * If false, a guide box is drawn to illustrate the new point size.
	 * <p>
	 * Default: true
	 */
	public boolean isLiveRedraw() {
		return getAttr(Attrs.liveRedraw, true).asBoolean();
	}

	/**
	 * Sets if updates points as they are dragged.
	 * If false, a guide box is drawn to illustrate the new point size.
	 */
	public void setLiveRedraw(boolean liveRedraw) {
		setAttr(Attrs.liveRedraw, liveRedraw, true);
	}

	/**
	 * Options for the drag handles.
	 *
	 * @author rudyhuang
	 * @since 7.2.1.0
	 */
	public static class DragHandle extends Optionable {
		private enum Attrs implements PlotAttribute, DynamicalAttribute {
			className, color, cursor, lineColor, lineWidth, pathFormatter, zIndex
		}

		/**
		 * Returns the class name of the drag handles.
		 * <p>
		 * Default: "highcharts-drag-handle"
		 */
		public String getClassName() {
			return getAttr(Attrs.className, "highcharts-drag-handle").asString();
		}

		/**
		 * Sets the class name of the drag handles.
		 */
		public void setClassName(String className) {
			setAttr(Attrs.className, className, "highcharts-drag-handle");
		}

		/**
		 * Returns the fill color of the drag handles.
		 * <p>
		 * Default: "#fff"
		 */
		public Color getColor() {
			if (!containsKey(Attrs.color)) {
				setColor("#fff");
			}
			return (Color) getAttr(Attrs.color);
		}

		/**
		 * Sets the fill color of the drag handles.
		 */
		public void setColor(Color color) {
			setAttr(Attrs.color, color, NOT_NULL_VALUE);
		}

		/**
		 * Sets the fill color of the drag handles.
		 */
		public void setColor(String color) {
			setColor(new Color(color));
		}

		/**
		 * Returns the mouse cursor to use for the drag handles.
		 * By default this is intelligently switching between <code>ew-resize</code> and
		 * <code>ns-resize</code> depending on the direction the point is being dragged.
		 * <p>
		 * Default: null
		 */
		public String getCursor() {
			return getAttr(Attrs.cursor, null).asString();
		}

		/**
		 * Sets the mouse cursor to use for the drag handles.
		 * By default this is intelligently switching between <code>ew-resize</code> and
		 * <code>ns-resize</code> depending on the direction the point is being dragged.
		 */
		public void setCursor(String cursor) {
			setAttr(Attrs.cursor, cursor);
		}

		/**
		 * Returns the line color of the drag handles.
		 * <p>
		 * Default: "rgba(0, 0, 0, 0.6)"
		 */
		public Color getLineColor() {
			if (!containsKey(Attrs.lineColor)) {
				setLineColor("rgba(0, 0, 0, 0.6)");
			}
			return (Color) getAttr(Attrs.lineColor);
		}

		/**
		 * Sets the line color of the drag handles.
		 */
		public void setLineColor(Color color) {
			setAttr(Attrs.lineColor, color, NOT_NULL_VALUE);
		}

		/**
		 * Sets the line color of the drag handles.
		 */
		public void setLineColor(String color) {
			setLineColor(new Color(color));
		}

		/**
		 * Returns the line width for the drag handles.
		 * <p>
		 * Default: 1.
		 */
		public Number getLineWidth() {
			return getAttr(Attrs.lineWidth, 1).asNumber();
		}

		/**
		 * Sets the line width for the drag handles.
		 */
		public void setLineWidth(Number lineWidth) {
			setAttr(Attrs.lineWidth, lineWidth, 1);
		}

		/**
		 * Returns the function to define the SVG path to use for the drag handles.
		 * Takes the point as argument. Should return an SVG path in array format.
		 * The SVG path is automatically positioned on the point.
		 * <p>
		 * Default: null
		 */
		public JavaScriptValue getPathFormatter() {
			return (JavaScriptValue) getAttr(Attrs.pathFormatter);
		}

		/**
		 * Sets the function to define the SVG path to use for the drag handles.
		 * Takes the point as argument. Should return an SVG path in array format.
		 * The SVG path is automatically positioned on the point.
		 */
		public void setPathFormatter(JavaScriptValue pathFormatter) {
			setAttr(Attrs.pathFormatter, pathFormatter);
		}

		/**
		 * Returns the z index for the drag handles.
		 * <p>
		 * Default: 901
		 */
		public int getZIndex() {
			return getAttr(Attrs.zIndex, 901).asInt();
		}

		/**
		 * Sets the z index for the drag handles.
		 */
		public void setZIndex(int zIndex) {
			setAttr(Attrs.zIndex, zIndex, 901);
		}
	}

	public static class GuideBox extends Optionable {

	}
}
