/* Funnel3DPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Thu Nov 21 10:01:28 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.Color;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.Series;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An option for funnel3d plot
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class Funnel3DPlotOptions extends FunnelPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		borderRadius,
		centerInCategory,
		edgeColor,
		edgeWidth, gradientForSides, grouping, groupPadding, groupZPadding,
		maxPointWidth, minPointLength, pointPadding, pointWidth
	}

	/**
	 * When true, the columns will center in the category, ignoring null or missing points.
	 * When false, space will be reserved for null or missing points.
	 *
	 * <p>
	 * Default: {@code false}.
	 * @since 10.2.1.0
	 */
	public boolean isCenterInCategory() {
		return getAttr(Attrs.centerInCategory, false).asBoolean();
	}

	/**
	 * When true, the columns will center in the category, ignoring null or missing points.
	 * When false, space will be reserved for null or missing points.
	 * @since 10.2.1.0
	 */
	public void setCenterInCategory(boolean centerInCategory) {
		setAttr(Attrs.centerInCategory, centerInCategory);
	}

	/**
	 * 3D columns only. Returns the width of the colored edges.
	 * <p>
	 * Default: 0
	 */
	public Number getEdgeWidth() {
		return getAttr(Attrs.edgeWidth, 0).asNumber();
	}

	/**
	 * 3D columns only. Sets the width of the colored edges.
	 */
	public void setEdgeWidth(Number edgeWidth) {
		setAttr(Attrs.edgeWidth, edgeWidth);
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public Color getEdgeColor() {
		return (Color) getAttr(Attrs.edgeColor, null).asValue();
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setEdgeColor(Color color) {
		setAttr(Attrs.edgeColor, color);
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setEdgeColor(String color) {
		setColor(new Color(color));
	}

	/**
	 * By default sides fill is set to a gradient through this option being set to true.
	 * Set to false to get solid color for the sides.
	 * <p>
	 * Default: true
	 */
	public boolean isGradientForSides() {
		return getAttr(Attrs.gradientForSides, true).asBoolean();
	}

	/**
	 * By default sides fill is set to a gradient through this option being set to true.
	 * Set to false to get solid color for the sides.
	 */
	public void setGradientForSides(boolean gradientForSides) {
		setAttr(Attrs.gradientForSides, gradientForSides);
	}

	/**
	 * Returns whether to group non-stacked columns or to let them render independent
	 * of each other. Non-grouped columns will be laid out individually and overlap each other.
	 * <p>Default: {@code true}
	 * @since 10.2.1.0
	 */
	public boolean isGrouping() {
		return getAttr(Attrs.grouping, true).asBoolean();
	}

	/**
	 * Sets whether to group non-stacked columns or to let them render independent
	 * of each other. Non-grouped columns will be laid out individually and overlap each other.
	 * <p>Default: {@code true}
	 * @since 10.2.1.0
	 */
	public void setGrouping(boolean grouping) {
		setAttr(Attrs.grouping, grouping);
	}

	/**
	 * Returns padding between each value groups, in x axis units.
	 * <p>Default: 0.2
	 * @since 10.2.1.0
	 */
	public Number getGroupPadding() {
		return getAttr(Attrs.groupPadding, 0.2).asNumber();
	}

	/**
	 * Sets padding between each value groups, in x axis units.
	 * @since 10.2.1.0
	 */
	public void setGroupPadding(Number groupPadding) {
		setAttr(Attrs.groupPadding, groupPadding);
	}

	/**
	 * Returns the spacing between columns on the Z Axis in a 3D chart.
	 * <p>
	 * Default: 1
	 */
	public Number getGroupZPadding() {
		return getAttr(Attrs.groupZPadding, 1).asNumber();
	}

	/**
	 * Sets the spacing between columns on the Z Axis in a 3D chart.
	 * <p>
	 * Default: 1
	 */
	public void setGroupZPadding(Number groupZPadding) {
		setAttr(Attrs.groupZPadding, groupZPadding);
	}

	/**
	 * Returns the maximum allowed pixel width for a column,
	 * translated to the height of a bar in a bar chart.
	 * This prevents the columns from becoming too wide when there is a small
	 * number of points in the chart.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public Number getMaxPointWidth() {
		return getAttr(Attrs.maxPointWidth, null).asNumber();
	}

	/**
	 * Sets the maximum allowed pixel width for a column,
	 * translated to the height of a bar in a bar chart.
	 * This prevents the columns from becoming too wide when there is a small
	 * number of points in the chart.
	 * @since 10.2.1.0
	 */
	public void setMaxPointWidth(Number maxPointWidth) {
		setAttr(Attrs.maxPointWidth, maxPointWidth);
	}

	/**
	 * Returns the minimal height for a column or width for a bar. By default, 0
	 * values are not shown. To visualize a 0 (or close to zero) point, set the
	 * minimal point length to a pixel value like 3. In stacked column charts,
	 * minPointLength might not be respected for tightly packed values.
	 * <p>Default: 0
	 * @since 10.2.1.0
	 */
	public Number getMinPointLength() {
		return getAttr(Attrs.minPointLength, 0).asNumber();
	}

	/**
	 * Sets the minimal height for a column or width for a bar. By default, 0
	 * values are not shown. To visualize a 0 (or close to zero) point, set the
	 * minimal point length to a pixel value like 3. In stacked column charts,
	 * minPointLength might not be respected for tightly packed values.
	 * @since 10.2.1.0
	 */
	public void setMinPointLength(Number minPointLength) {
		setAttr(Attrs.minPointLength, minPointLength, 0);
	}

	/**
	 * Returns padding between each column or bar, in x axis units.
	 * <p> Default: {@code 0.1}
	 * @since 10.2.1.0
	 */
	public Number getPointPadding() {
		return getAttr(Attrs.pointPadding, 0.1).asNumber();
	}

	/**
	 * Sets padding between each column or bar, in x axis units. 'pointPadding
	 * padding between each column or bar, in x axis units.
	 * @since 10.2.1.0
	 */
	public void setPointPadding(Number pointPadding) {
		setAttr(Attrs.pointPadding, pointPadding);
	}

	/**
	 * Returns a pixel value specifying a fixed width for each column or bar.
	 * When <code>null</code>, the width is calculated from the
	 * <code>pointPadding</code> and <code>groupPadding</code>.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public Number getPointWidth() {
		return getAttr(Attrs.pointWidth, null).asNumber();
	}

	/**
	 * Sets a pixel value specifying a fixed width for each column or bar. When
	 * <code>null</code>, the width is calculated from the
	 * <code>pointPadding</code> and <code>groupPadding</code>.
	 * @since 10.2.1.0
	 */
	public void setPointWidth(Number pointWidth) {
		setAttr(Attrs.pointWidth, pointWidth);
	}
}
