/* ScatterPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Thu, Jan 16, 2014  3:58:41 PM, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.

 */
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.Cluster;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An option for scatter plot
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author RaymondChao
 */
public class ScatterPlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute {
		cluster,
		depth, lineWidth, height, jitter, stickyTracking, width
	}

	/**
	 * Returns the options for marker clusters, the concept of sampling the data values
	 * into larger blocks in order to ease readability and increase performance of
	 * the JavaScript charts.
	 * <p> Default: an instance of Cluster.
	 * @since 10.2.1.0
	 */
	public Cluster getCluster() {
		return (Cluster) getAttr(Attrs.cluster, Cluster.class).asValue();
	}

	/**
	 * Sets the options for marker clusters, the concept of sampling the data values
	 * into larger blocks in order to ease readability and increase performance of
	 * the JavaScript charts.
	 * <p> Default: an instance of Cluster.
	 * @since 10.2.1.0
	 */
	public void setCluster(Cluster cluster) {
		setAttr(Attrs.cluster, cluster);
	}

	/**
	 * Returns the total depth of the chart.
	 * <p>
	 * Default: null.
	 */
	public Number getDepth() {
		return getAttr(Attrs.depth, null).asNumber();
	}

	/**
	 * Sets the total depth of the chart.
	 * 
	 * @param depth
	 *            the total depth of the chart. Defaults to 100.
	 */
	public void setDepth(Number depth) {
		setAttr(Attrs.depth, depth);
	}
	
	/**
	 * Returns the height of the scatter plot.
	 * <p>
	 * Default: null.
	 * 
	 * @return the height of the scatter plot.
	 */
	public Number getHeight() {
		return getAttr(Attrs.height, null).asNumber();
	}

	/**
	 * Sets the height of the scatter plot.
	 */
	public void setHeight(Number height) {
		setAttr(Attrs.height, height);
	}

	/**
	 * Apply a jitter effect for the rendered markers.
	 * When plotting discrete values, a little random noise may help telling the points apart.
	 * The jitter setting applies a random displacement of up to n axis units in either direction.
	 * So for example on a horizontal X axis, setting the jitter.x to 0.24 will
	 * render the point in a random position between 0.24 units to the left
	 * and 0.24 units to the right of the true axis position.
	 * On a category axis, setting it to 0.5 will fill up the bin and make the data appear continuous.
	 *
	 * When rendered on top of a box plot or a column series,
	 * a jitter value of 0.24 will correspond to the underlying series' default groupPadding and pointPadding settings.
	 * @since 7.2.1.0
	 */
	public Jitter getJitter() {
		Jitter jitter = (Jitter) getAttr(Attrs.jitter);
		if (jitter == null) {
			jitter = new Jitter();
			setJitter(jitter);
		}
		return jitter;
	}

	/**
	 * Apply a jitter effect for the rendered markers.
	 * When plotting discrete values, a little random noise may help telling the points apart.
	 * The jitter setting applies a random displacement of up to n axis units in either direction.
	 * So for example on a horizontal X axis, setting the jitter.x to 0.24 will
	 * render the point in a random position between 0.24 units to the left
	 * and 0.24 units to the right of the true axis position.
	 * On a category axis, setting it to 0.5 will fill up the bin and make the data appear continuous.
	 *
	 * When rendered on top of a box plot or a column series,
	 * a jitter value of 0.24 will correspond to the underlying series' default groupPadding and pointPadding settings.
	 * @since 7.2.1.0
	 */
	public void setJitter(Jitter jitter) {
		setAttr(Attrs.jitter, jitter);
	}

	/**
	 * Returns the width of the line connecting the data points.
	 * <p>Default: 0
	 */
	public Number getLineWidth() {
		return getAttr(Attrs.lineWidth, 0).asNumber();
	}

	/**
	 * Sets the width of the line connecting the data points. 'lineWidth the
	 * width of the line connecting the data points.
	 */
	public void setLineWidth(Number lineWidth) {
		setAttr(Attrs.lineWidth, lineWidth, 0);
	}

	/**
	 * Returns sticky tracking of mouse events. When true, the
	 * <code>mouseOut</code> event on a series isn't triggered until the mouse
	 * moves over another series, or out of the plot area. When false, the
	 * <code>mouseOut</code> event on a series is triggered when the mouse
	 * leaves the area around the series' graph or markers. This also implies
	 * the tooltip. When <code>stickyTracking</code> is false and
	 * <code>tooltip.shared</code> is false, the tooltip will be hidden when
	 * moving the mouse between series.
	 */
	public boolean isStickyTracking() {
		return getAttr(Attrs.stickyTracking, false).asBoolean();
	}

	/**
	 * Sets sticky tracking of mouse events. When true, the
	 * <code>mouseOut</code> event on a series isn't triggered until the mouse
	 * moves over another series, or out of the plot area. When false, the
	 * <code>mouseOut</code> event on a series is triggered when the mouse
	 * leaves the area around the series' graph or markers. This also implies
	 * the tooltip. When <code>stickyTracking</code> is false and
	 * <code>tooltip.shared</code> is false, the tooltip will be hidden when
	 * moving the mouse between series.
	 */
	public void setStickyTracking(boolean stickyTracking) {
		setAttr(Attrs.stickyTracking, stickyTracking);
	}
	
	/**
	 * Returns the width of the scatter plot.
	 * <p>
	 * Default: null.
	 * 
	 * @return the width of the scatter plot.
	 */
	public Number getWidth() {
		return getAttr(Attrs.width, null).asNumber();
	}

	/**
	 * Sets the width of the scatter plot.
	 */
	public void setWidth(Number width) {
		setAttr(Attrs.width, width);
	}

}